# Copyright (C) 2021 Red Hat, Inc., Pavel Moravec <pmoravec@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import (Plugin, RedHatPlugin, DebianPlugin,
                                UbuntuPlugin)


class ForemanInstaller(Plugin, DebianPlugin, UbuntuPlugin):

    short_desc = 'Foreman installer and maintainer'

    plugin_name = 'foreman_installer'
    profiles = ('sysmgmt',)
    packages = ('foreman-installer', 'rubygem-foreman_maintain')

    def setup(self):
        self.add_copy_spec([
            "/etc/foreman-installer/*",
            "/var/log/foreman-installer/*",
            "/var/log/foreman-maintain/*",
            "/var/lib/foreman-maintain/data.yml",
            "/var/lib/foreman-maintain/satellite_metrics.yml",
            "/etc/foreman-maintain/foreman_maintain.yml",
            # specifically collect .applied files
            # that would be skipped otherwise as hidden files
            "/etc/foreman-installer/scenarios.d/*/.applied",
        ])

        # skip collecting individual migration scripts;
        # .applied file in each dir is still
        self.add_forbidden_path(
                "/etc/foreman-installer/scenarios.d/*.migrations/*.rb"
        )

        self.add_cmd_output([
            'foreman-maintain service status',
        ])

    def postproc(self):
        install_logs = "/var/log/foreman-installer/"
        logsreg = r"((foreman.*)?(\"::(foreman(.*?)|katello).*)?((::(.*)::.*" \
                  r"(passw|cred|token|secret|key).*(\")?:)|(storepass )" \
                  r"|(password =)))(.*)"
        self.do_path_regex_sub(install_logs, logsreg, r"\1 ********")
        # need to do two passes here, debug output has different formatting
        logs_debug_reg = (r"(\s)+(Found key: (\"(foreman(.*?)|katello)"
                          r"::(.*(token|secret|key|passw).*)\") value:) "
                          r"(.*)")
        self.do_path_regex_sub(install_logs, logs_debug_reg, r"\1 \2 ********")
        # also hide passwords in yet different formats
        self.do_path_regex_sub(
            install_logs,
            r"((\.|_|-)password(=\'|=|\", \"))(\w*)",
            r"\1********")
        self.do_path_regex_sub(
            "/var/log/foreman-installer/foreman-proxy*",
            r"(\s*proxy_password\s=) (.*)",
            r"\1 ********")
        self.do_path_regex_sub(
            "/var/log/foreman-maintain/foreman-maintain.log*",
            r"(((passw|cred|token|secret)=)|(password ))(.*)",
            r"\1********")
        # all scrubbing applied to configs must be applied to installer logs
        # as well, since logs contain diff of configs
        self.do_path_regex_sub(
            r"(/etc/foreman-(installer|maintain)/(.*)((conf)(.*)?))|"
            fr"({install_logs})",
            r"((\:|\s*)(passw|cred|token|secret|key).*(\:\s|=))(.*)",
            r"\1********")
        # yaml values should be alphanumeric
        self.do_path_regex_sub(
            r"(/etc/foreman-(installer|maintain)/(.*)((yaml|yml)(.*)?))|"
            fr"({install_logs})",
            r"((\:|\s*)(passw|cred|token|secret|key).*(\:\s|=))(.*)",
            r'\1"********"')


# Add Red Hat Insights tags for RedHatPlugin only

class RedHatForemanInstaller(ForemanInstaller, RedHatPlugin):

    def setup(self):

        self.add_file_tags({
            '/var/log/foreman-installer/satellite.log':
                ['foreman_satellite_log', 'satellite_installer_log'],
            '/var/log/foreman-installer/capsule.log':
                ['capsule_log', 'capsule_installer_log'],
        })

        super().setup()


# vim: set et ts=4 sw=4 :



# Copyright (C) 2019 Red Hat, Inc., Jake Hunsaker <jhunsake@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin


class Rpmostree(Plugin, RedHatPlugin):

    short_desc = 'rpm-ostree image/package system'

    plugin_name = 'rpmostree'
    packages = ('rpm-ostree',)

    def setup(self):
        self.add_copy_spec('/etc/ostree/remotes.d/')

        subcmds = [
            'status --json',
            'kargs',
            'db list',
            'db diff',
            '--version'
        ]

        self.add_cmd_output([f"rpm-ostree {subcmd}" for subcmd in subcmds])

        units = [
            'rpm-ostreed',
            'rpm-ostreed-automatic',
            'rpm-ostree-bootstatus'
        ]
        for unit in units:
            self.add_journal(unit)

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

import os
from sos.report.plugins import Plugin, RedHatPlugin, PluginOpt


class Oratab(Plugin, RedHatPlugin):
    short_desc = 'Oratab discovery for Oracle instances on Linux'

    plugin_name = "oratab"
    profiles = ('services',)
    packages = ('oratab',)

    option_list = [
        PluginOpt('oratab', default='/etc/oratab', val_type=str,
                  desc='location of the oratab file')
    ]

    def setup(self):
        ora_tab = self.get_option('oratab')

        if os.path.isfile(ora_tab):
            try:
                # Since the oratab file can have multiple unique SID
                # entries that point to the same oracle home directory,
                # don't gather duplicate files
                path_list = set()

                with open(ora_tab, 'r', encoding='UTF-8') as ofile:
                    for line in ofile.read().splitlines():
                        if line.startswith('#') or not line.strip():
                            continue
                        path_list.add(line.split(':')[1])

                dbfiles = [
                    'dbs/init*.ora',
                    'dbs/spfile*.ora'
                ]

                self.add_copy_spec([
                    self.path_join(path, dbfile) for dbfile in dbfiles
                    for path in path_list])
            except IOError as ex:
                self._log_error(f'Could not open conf file {ora_tab}: {ex}')
                return
        else:
            self._log_warn("Could not locate oratab file. "
                           "Oracle data will not be collected.")

# vim: set et ts=4 sw=4 :



# Copyright (c) 2017 Bryan Quigley <bryan.quigley@canonical.com>
# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

import re
from sos.report.plugins import Plugin, IndependentPlugin


class Snap(Plugin, IndependentPlugin):

    short_desc = 'Snap packages'

    plugin_name = 'snap'
    profiles = ('system', 'sysmgmt', 'packagemanager')
    packages = ('snapd',)
    services = ('snapd',)

    def setup(self):
        self.add_copy_spec("/var/lib/snapd/state.json")
        self.add_cmd_output("snap list --all", root_symlink="installed-snaps")
        self.add_cmd_output([
            "snap --version",
            "snap version",
            "snap whoami",
            "snap model --verbose",
            "snap model --serial --verbose",
            "snap services",
            "snap connections",
            "snap changes --abs-time",
            "snap validate",
            "snap debug state --abs-time --changes /var/lib/snapd/state.json",
            "snap debug stacktraces",
            "snap get system -d",
        ])

        all_pkgs = self.policy.package_manager.packages

        for pkg_name in all_pkgs:
            pkg = self.policy.package_manager.pkg_by_name(pkg_name)
            if pkg['pkg_manager'] == 'snap':
                self.add_cmd_output(f"snap connections {pkg['name']}")

        self.add_cmd_output("snap debug connectivity", timeout=10)

        # If we have gadget snaps, then we collect more files, this is
        # typically defined in the Notes column
        snap_list = self.exec_cmd('snap list')

        if snap_list['status'] == 0:
            output = snap_list['output']

            for line in output.splitlines()[1:]:
                if line == "":
                    continue
                snap_pkg = line.split()
                if re.match(r".*gadget.*$", snap_pkg[5]):
                    self.add_copy_spec([
                        f"/snap/{snap_pkg[0]}/current/meta/gadget.yaml",
                    ])

        snap_changes = self.collect_cmd_output('snap changes')

        if snap_changes['status'] == 0:
            output = snap_changes['output']

            for line in output.splitlines()[1:]:
                if line == "":
                    continue
                change = line.split()
                change_id, change_status = change[0], change[1]
                if change_status in ("Doing", "Error"):
                    self.add_cmd_output(f"snap tasks {change_id} --abs-time")

    def postproc(self):
        self.do_file_sub(
            "/var/lib/snapd/state.json",
            (r"\"(macaroon|store-macaroon|key-id|session-macaroon|macaroon-key"
                r"|store-discharges)\":\"?\[?([\"?A-Za-z0-9_=\-,]*)\"?\]?"),
            r'"\1":"***"')

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, IndependentPlugin


class Cifs(Plugin, IndependentPlugin):

    short_desc = 'SMB file system information'
    plugin_name = 'cifs'
    profiles = ('storage', 'network', 'cifs')
    packages = ('cifs-utils',)

    def setup(self):
        self.add_forbidden_path([
            "/proc/fs/cifs/traceSMB",
            "/proc/fs/cifs/cifsFYI",
        ])

        self.add_copy_spec([
            "/etc/request-key.d/cifs.spnego.conf",
            "/etc/request-key.d/cifs.idmap.conf",
            "/proc/keys",
            "/proc/fs/cifs/*",
        ])

# vim: set et ts=4 sw=4 :



# Copyright (C) 2019 Red Hat, Inc., Jake Hunsaker <jhunsake@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin


class Leapp(Plugin, RedHatPlugin):

    short_desc = 'Leapp upgrade handling tool'

    plugin_name = 'leapp'
    packages = ('leapp',)
    files = ('/var/lib/leapp',)

    def setup(self):
        self.add_copy_spec([
            '/etc/migration-results',
            '/etc/leapp/actor_cond.d',
            '/etc/leapp/files/dnf.conf',
            '/var/log/leapp/answerfile',
            '/var/log/leapp/dnf-debugdata/',
            '/var/log/leapp/leapp-preupgrade.log',
            '/var/log/leapp/leapp-upgrade.log',
            '/var/log/leapp/leapp-report.txt',
            '/var/log/leapp/dnf-plugin-data.txt'
        ])

        # capture DB without sizelimit
        self.add_copy_spec('/var/lib/leapp/leapp.db', sizelimit=0)

        self.add_cmd_output('leapp --version')

    def postproc(self):
        # Scrub password and proxy_password likewise from dnf.conf
        self.do_file_sub("/etc/leapp/files/dnf.conf",
                         r"(password(\s)*=(\s)*)(\S+)\n",
                         r"\1********\n")

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, IndependentPlugin


class Hyperv(Plugin, IndependentPlugin):
    """Hyper-V client information"""

    short_desc = 'Microsoft Hyper-V client information'
    plugin_name = "hyperv"
    files = ('/sys/bus/vmbus/',)

    def setup(self):

        self.add_copy_spec([
            "/sys/bus/vmbus/drivers/",
            # copy devices/*/* instead of devices/ to follow link files
            "/sys/bus/vmbus/devices/*/*"
        ])

        self.add_cmd_output("lsvmbus -vv")

# vim: set et ts=4 sw=4 :




import logging
import os
import platform
import time
import json
import tempfile
import random
import string
import sys

from pwd import getpwuid
from textwrap import fill
from sos.presets import (NO_PRESET, GENERIC_PRESETS, PRESETS_PATH,
                         PresetDefaults, DESC, NOTE, OPTS)
from sos.policies.package_managers import PackageManager
from sos.utilities import (ImporterHelper, import_module, get_human_readable,
                           bold)
from sos.report.plugins import IndependentPlugin, ExperimentalPlugin
from sos.options import SoSOptions
from sos import _sos as _


def import_policy(name):
    policy_fqname = f"sos.policies.distros.{name}"
    try:
        return import_module(policy_fqname, Policy)
    except ImportError:
        return None


def load(cache={}, sysroot=None, init=None, probe_runtime=True,
         remote_exec=None, remote_check=''):
    if 'policy' in cache:
        return cache.get('policy')

    import sos.policies.distros
    helper = ImporterHelper(sos.policies.distros)
    for module in helper.get_modules():
        for policy in import_policy(module):
            if policy.check(remote=remote_check):
                cache['policy'] = policy(sysroot=sysroot, init=init,
                                         probe_runtime=probe_runtime,
                                         remote_exec=remote_exec)
                break

    if sys.platform != 'linux':
        raise Exception("SoS is not supported on this platform")

    if 'policy' not in cache:
        cache['policy'] = sos.policies.distros.GenericLinuxPolicy()

    return cache['policy']


class Policy():
    """Policies represent distributions that sos supports, and define the way
    in which sos behaves on those distributions. A policy should define at
    minimum a way to identify the distribution, and a package manager to allow
    for package based plugin enablement.

    Policies also control preferred ContainerRuntime()'s, upload support to
    default locations for distribution vendors, disclaimer text, and default
    presets supported by that distribution or vendor's products.

    Every Policy will also need at least one "tagging class" for plugins.

    :param sysroot: Set the sysroot for the system, if not /
    :type sysroot: ``str`` or ``None``

    :param probe_runtime: Should the Policy try to load a ContainerRuntime
    :type probe_runtime: ``bool``

    :param remote_exec:     If this policy is loaded for a remote node, use
                            this to facilitate executing commands via the
                            SoSTransport in use
    :type remote_exec:      ``SoSTranport.run_command()``

    :cvar os_release_name: The name of the distribution as it appears in the
                           os-release (-esque) file for the NAME variable.
    :vartype os_release_name: ``str``

    :cvar os_release_id: The ID variable to match in a distribution's release
                         file.
    :vartype os_release_id: ``str``

    :cvar os_release_file: The filepath of the distribution's os-release file
    :vartype os_release_file: ``str``

    :cvar vendor: The name of the vendor producing the distribution
    :vartype vendor: ``str``

    :cvar vendor_urls: List of URLs for the vendor's website, or support portal
    :vartype vendor_urls: ``list`` of ``tuples`` formatted
        ``(``description``, ``url``)``

    :cvar vendor_text: Additional text to add to the banner message
    :vartype vendor_text: ``str``

    :cvar name_pattern: The naming pattern to be used for naming archives
                        generated by sos. Values of `legacy`, and `friendly`
                        are preset patterns. May also be set to an explicit
                        custom pattern, see `get_archive_name()`
    :vartype name_pattern: ``str``
    """

    msg = _("""\
This command will collect system configuration and diagnostic information \
from this %(os_release_name)s system.

For more information on %(vendor)s visit:

  %(vendor_urls)s

The generated archive may contain data considered sensitive and its content \
should be reviewed by the originating organization before being passed to \
any third party.

%(changes_text)s

%(vendor_text)s
""")
    os_release_name = 'Unknown'
    os_release_file = ''
    os_release_id = ''
    vendor = "Unknown"
    vendor_urls = [('Example URL', "http://www.example.com/")]
    vendor_text = ""
    PATH = "/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin"
    name_pattern = 'legacy'
    presets = {"": PresetDefaults()}
    presets_path = PRESETS_PATH
    _in_container = False

    def __init__(self, sysroot=None, probe_runtime=True, remote_exec=None):
        """Subclasses that choose to override this initializer should call
        super() to ensure that they get the required platform bits attached.
        super(SubClass, self).__init__(). Policies that require runtime
        tests to construct PATH must call self.set_exec_path() after
        modifying PATH in their own initializer."""
        self.soslog = logging.getLogger('sos')
        self.ui_log = logging.getLogger('sos_ui')
        self._parse_uname()
        self.case_id = None
        self.probe_runtime = probe_runtime
        self.package_manager = PackageManager()
        self.valid_subclasses = [IndependentPlugin]
        self.remote_exec = remote_exec
        if not self.remote_exec:
            self.set_exec_path()
        self.sysroot = sysroot
        self.register_presets(GENERIC_PRESETS)

    @classmethod
    def check(cls, remote=''):
        """
        This function is responsible for determining if the underlying system
        is supported by this policy.

        If `remote` is provided, it should be the contents of os-release from
        a remote host, or a similar vendor-specific file that can be used in
        place of a locally available file.

        :returns: ``True`` if the Policy should be loaded, else ``False``
        :rtype: ``bool``
        """
        raise NotImplementedError

    @property
    def forbidden_paths(self):
        """This property is used to determine the list of forbidden paths
        set by the policy. Note that this property will construct a
        *cumulative* list based on all subclasses of a given policy.

        :returns: All patterns of policy forbidden paths
        :rtype: ``list``
        """
        if not hasattr(self, '_forbidden_paths'):
            self._forbidden_paths = []
            for cls in self.__class__.__mro__:
                if hasattr(cls, 'set_forbidden_paths'):
                    self._forbidden_paths.extend(cls.set_forbidden_paths())
        return list(set(self._forbidden_paths))

    @classmethod
    def set_forbidden_paths(cls):
        """Use this to *append* policy-specifc forbidden paths that apply to
        all plugins. Setting this classmethod on an invidual policy will *not*
        override subclass-specific paths
        """
        return [
            '*.egg',
            '*.pyc',
            '*.pyo',
            '*.swp'
        ]

    def in_container(self):
        """Are we running inside a container?

        :returns: ``True`` if in a container, else ``False``
        :rtype: ``bool``
        """
        return self._in_container

    def dist_version(self):
        """
        Return the OS version
        """

    def get_preferred_archive(self):
        """
        Return the class object of the prefered archive format for this
        platform
        """
        from sos.archive import TarFileArchive
        return TarFileArchive

    def get_archive_name(self):
        """
        This function should return the filename of the archive without the
        extension.

        This uses the policy's `name_pattern` attribute to determine the name.
        There are two pre-defined naming patterns - `legacy` and `friendly`
        that give names like the following:

        * legacy - `sosreport-tux.123456-20171224185433`
        * friendly - `sosreport-tux-mylabel-123456-2017-12-24-ezcfcop.tar.xz`

        A custom name_pattern can be used by a policy provided that it
        defines name_pattern using a format() style string substitution.

        Usable substitutions are:

            * name  - the short hostname of the system
            * label - the label given by --label
            * case  - the case id given by --case-id
            * rand  - a random string of 7 alpha characters

        Note that if a datestamp is needed, the substring should be set
        in `name_pattern` in the format accepted by ``strftime()``.

        :returns: A name to be used for the archive, as expanded from
                  the Policy `name_pattern`
        :rtype: ``str``
        """
        name = self.get_local_name().split('.')[0]  # pylint: disable=no-member
        case = self.case_id
        label = self.commons['cmdlineopts'].label
        date = ''
        rand = ''.join(random.choice(string.ascii_lowercase) for x in range(7))

        if self.name_pattern == 'legacy':
            case = '.' + case if case else ''
            date = '-%Y%m%d%H%M%S'
            nstr = f"sosreport-{name}{case}{date}"
        elif self.name_pattern == 'friendly':
            case = '-' + case if case else ''
            label = '-' + label if label else ''
            date = '-%Y-%m-%d'
            nstr = f"sosreport-{name}{label}{case}{date}-{rand}"
        else:
            nstr = self.name_pattern

        # pylint: disable-next=no-member
        return self.sanitize_filename(time.strftime(nstr))

    # for some specific binaries like "xz", we need to determine package
    # providing it; that is policy specific. By default return the binary
    # name itself until particular policy overwrites it
    def _get_pkg_name_for_binary(self, binary):
        return binary

    def get_tmp_dir(self, opt_tmp_dir):
        if not opt_tmp_dir:
            return tempfile.gettempdir()
        return opt_tmp_dir

    def match_plugin(self, plugin_classes):
        """Determine what subclass of a Plugin should be used based on the
        tagging classes assigned to the Plugin

        :param plugin_classes: The classes that the Plugin subclasses
        :type plugin_classes: ``list``

        :returns: The first tagging class that matches one of the Policy's
                  `valid_subclasses`
        :rtype: ``PluginDistroTag``
        """
        if len(plugin_classes) > 1:
            for p in plugin_classes:
                # Give preference to the first listed tagging class
                # so that e.g. UbuntuPlugin is chosen over DebianPlugin
                # on an Ubuntu installation.
                if issubclass(p, self.valid_subclasses[0]):
                    return p
        return plugin_classes[0]

    def validate_plugin(self, plugin_class, experimental=False):
        """
        Verifies that the plugin_class should execute under this policy

        :param plugin_class: The tagging class being checked
        :type plugin_class: ``PluginDistroTag``

        :returns: ``True`` if the `plugin_class` is allowed by the policy
        :rtype: ``bool``
        """
        valid_subclasses = [IndependentPlugin] + self.valid_subclasses
        if experimental:
            valid_subclasses += [ExperimentalPlugin]
        return any(issubclass(plugin_class, class_) for
                   class_ in valid_subclasses)

    def pre_work(self):
        """
        This function is called prior to collection.
        """

    def post_work(self):
        """
        This function is called after the sos report has been generated.
        """

    def pkg_by_name(self, pkg):
        """Wrapper to retrieve a package from the Policy's package manager

        :param pkg: The name of the package
        :type pkg: ``str``

        :returns: The first package that matches `pkg`
        :rtype: ``str``
        """
        return self.package_manager.pkg_by_name(pkg)

    def _parse_uname(self):
        (system, node, release,
         version, machine, _) = platform.uname()
        self.system = system
        self.hostname = node
        self.release = release
        self.smp = version.split()[1] == "SMP"
        self.machine = machine

    def set_commons(self, commons):
        """Set common host data for the Policy to reference
        """
        self.commons = commons

    def _set_PATH(self, path):
        os.environ['PATH'] = path

    def set_exec_path(self):
        self._set_PATH(self.PATH)

    def is_root(self):
        """This method should return true if the user calling the script is
        considered to be a superuser

        :returns: ``True`` if user is superuser, else ``False``
        :rtype: ``bool``
        """
        return os.getuid() == 0

    def get_preferred_hash_name(self):
        """Returns the string name of the hashlib-supported checksum algorithm
        to use"""
        return "sha256"

    @classmethod
    def display_help(cls, section):
        section.set_title('SoS Policies')
        section.add_text(
            'Policies help govern how SoS operates on across different distri'
            'butions of Linux. They control aspects such as plugin enablement,'
            ' $PATH determination, how/which package managers are queried, '
            'default upload specifications, and more.'
        )

        section.add_text(
            "When SoS intializes most functions, for example "
            f"{bold('sos report')} and {bold('sos collect')}, one "
            "of the first operations is to determine the correct policy to "
            "load for the local system. Policies will determine the proper "
            "package manager to use, any applicable container runtime(s), and "
            "init systems so that SoS and report plugins can properly function"
            " for collections. Generally speaking a single policy will map to"
            " a single distribution; for example there are separate policies "
            "for Debian, Ubuntu, RHEL, and Fedora."
        )

        section.add_text(
            "It is currently not possible for users to directly control which "
            "policy is loaded."
        )

        pols = {
            'policies.cos': 'The Google Cloud-Optimized OS distribution',
            'policies.debian': 'The Debian distribution',
            'policies.redhat': ('Red Hat family distributions, not necessarily'
                                ' including forks'),
            'policies.ubuntu': 'Ubuntu/Canonical distributions'
        }

        seealso = section.add_section('See Also')
        seealso.add_text(
            "For more information on distribution policies, see below\n"
        )
        for pol, value in pols.items():
            seealso.add_text(f"{' ':>8}{pol:<20}{value:<30}", newline=False)

    def display_results(self, archive, directory, checksum, archivestat=None,
                        map_file=None):
        """Display final information about a generated archive

        :param archive: The name of the archive that was generated
        :type archive: ``str``

        :param directory: The build directory for sos if --build was used
        :type directory: ``str``

        :param checksum: The checksum of the archive
        :type checksum: ``str``

        :param archivestat: stat() information for the archive
        :type archivestat: `os.stat_result`

        :param map_file: If sos clean was invoked, the location of the mapping
                         file for this run
        :type map_file: ``str``
        """
        # Logging is shut down, but there are some edge cases where automation
        # does not capture printed output (e.g. avocado CI). Use the ui_log to
        # still print to console in this case.

        # make sure a report exists
        if not archive and not directory:
            return False

        if map_file:
            self.ui_log.info(
                _(f"\nA mapping of obfuscated elements is available at"
                  f"\n\t{map_file}")
            )

        if archive:
            self.ui_log.info(
                _(f"\nYour sos report has been generated and saved in:"
                  f"\n\t{archive}\n")
            )
            self.ui_log.info(
                _(f" Size\t{get_human_readable(archivestat.st_size)}")
            )
            self.ui_log.info(
                _(f" Owner\t{getpwuid(archivestat.st_uid).pw_name}")
            )
        else:
            self.ui_log.info(
                _(f"Your sos report build tree has been generated in:"
                  f"\n\t{directory}\n")
            )
        if checksum:
            self.ui_log.info(f" {self.get_preferred_hash_name()}\t{checksum}")
            self.ui_log.info(
                _("\nPlease send this file to your support representative.\n")
            )
        return None

    def get_msg(self):
        """This method is used to prepare the preamble text to display to
        the user in non-batch mode. If your policy sets self.os_release_name,
        that text will be substituted accordingly. You can also override this
        method to do something more complicated.

        :returns: Formatted banner message string
        :rtype: ``str``
        """
        if self.commons['cmdlineopts'].allow_system_changes:
            changes_text = "Changes CAN be made to system configuration."
        else:
            changes_text = "No changes will be made to system configuration."
        width = 72
        _msg = self.msg % {'os_release_name': self.os_release_name,
                           'vendor': self.vendor,
                           'vendor_urls': self._fmt_vendor_urls(),
                           'vendor_text': self.vendor_text,
                           'tmpdir': self.commons['tmpdir'],
                           'changes_text': changes_text}
        _fmt = ""
        for line in _msg.splitlines():
            _fmt = _fmt + fill(line, width, replace_whitespace=False) + '\n'
        return _fmt

    def _fmt_vendor_urls(self):
        """Formats all items in the ``vendor_urls`` class attr into a usable
        string for the banner message.

        :returns:   Formatted string of URLS
        :rtype:     ``str``
        """
        width = max(len(v[0]) for v in self.vendor_urls)
        return "\n".join(
            f"\t{url[0]:<{width}} : {url[1]}" for url in self.vendor_urls
        )

    def register_presets(self, presets, replace=False):
        """Add new presets to this policy object.

            Merges the presets dictionary ``presets`` into this ``Policy``
            object, or replaces the current presets if ``replace`` is
            ``True``.

            ``presets`` should be a dictionary mapping ``str`` preset names
            to ``<class PresetDefaults>`` objects specifying the command
            line defaults.

            :param presets: dictionary of presets to add or replace
            :param replace: replace presets rather than merge new presets.
        """
        if replace:
            self.presets = {}
        self.presets.update(presets)

    def find_preset(self, preset):
        """Find a preset profile matching the specified preset string.

            :param preset: a string containing a preset profile name.
            :returns: a matching PresetProfile.
        """
        # FIXME: allow fuzzy matching?
        for match, value in self.presets.items():
            if match == preset:
                return value

        return None

    def probe_preset(self):
        """Return a ``PresetDefaults`` object matching the runing host.

            Stub method to be implemented by derived policy classes.

            :returns: a ``PresetDefaults`` object.
        """
        return self.presets[NO_PRESET]

    def load_presets(self, presets_path=None):
        """Load presets from disk.

            Read JSON formatted preset data from the specified path,
            or the default location at ``/etc/sos/presets.d``.

            :param presets_path: a directory containing JSON presets.
        """
        presets_path = presets_path or self.presets_path
        if not os.path.exists(presets_path):
            return
        for preset_path in os.listdir(presets_path):
            preset_path = os.path.join(presets_path, preset_path)

            with open(preset_path, encoding='utf-8') as pf:
                try:
                    preset_data = json.load(pf)
                except ValueError:
                    continue

            for preset in preset_data.keys():
                pd = PresetDefaults(preset, opts=SoSOptions())
                data = preset_data[preset]
                pd.desc = data[DESC] if DESC in data else ""
                pd.note = data[NOTE] if NOTE in data else ""

                if OPTS in data:
                    for arg in data[OPTS]:
                        setattr(pd.opts, arg, data[OPTS][arg])
                pd.builtin = False
                self.presets[preset] = pd

    def add_preset(self, name=None, desc=None, note=None, opts=SoSOptions()):
        """Add a new on-disk preset and write it to the configured
            presets path.

            :param preset: the new PresetDefaults to add
        """
        presets_path = self.presets_path

        if not name:
            raise ValueError("Preset name cannot be empty")

        if name in self.presets:
            raise ValueError(f"A preset with name '{name}' already exists")

        preset = PresetDefaults(name=name, desc=desc, note=note, opts=opts)
        preset.builtin = False
        self.presets[preset.name] = preset
        preset.write(presets_path)

    def del_preset(self, name=""):
        if not name or name not in self.presets:
            raise ValueError(f"Unknown profile: '{name}'")

        preset = self.presets[name]

        if preset.builtin:
            raise ValueError(f"Cannot delete built-in preset '{preset.name}'")

        preset.delete(self.presets_path)
        self.presets.pop(name)


# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

import unittest
from ipaddress import ip_interface
from os.path import join

import sos.policies
from sos.cleaner.parsers.ip_parser import SoSIPParser
from sos.cleaner.parsers.mac_parser import SoSMacParser
from sos.cleaner.parsers.hostname_parser import SoSHostnameParser
from sos.cleaner.parsers.keyword_parser import SoSKeywordParser
from sos.cleaner.parsers.ipv6_parser import SoSIPv6Parser
from sos.cleaner.parsers.username_parser import SoSUsernameParser
from sos.cleaner.mappings.ip_map import SoSIPMap
from sos.cleaner.mappings.mac_map import SoSMacMap
from sos.cleaner.mappings.hostname_map import SoSHostnameMap
from sos.cleaner.mappings.keyword_map import SoSKeywordMap
from sos.cleaner.mappings.ipv6_map import SoSIPv6Map
from sos.cleaner.preppers import SoSPrepper
from sos.cleaner.preppers.hostname import HostnamePrepper
from sos.cleaner.preppers.ip import IPPrepper
from sos.cleaner.archives.sos import SoSReportArchive
from sos.options import SoSOptions


class CleanerMapTests(unittest.TestCase):

    def setUp(self):
        workdir = join(sos.policies.load().get_tmp_dir(None),
                       'sos_avocado_testing')
        self.mac_map = SoSMacMap(workdir)
        self.ip_map = SoSIPMap(workdir)
        self.host_map = SoSHostnameMap(workdir)
        self.host_map.sanitize_item('redhat.com')
        self.kw_map = SoSKeywordMap(workdir)
        self.ipv6_map = SoSIPv6Map(workdir)
        # required for test_hostname_concurrently* tests
        self.host_parser = SoSHostnameParser(config={}, workdir=workdir)
        self.host_parser.mapping.add('foobar.com')

    def test_mac_map_obfuscate_valid_v4(self):
        _test = self.mac_map.get('12:34:56:78:90:ab')
        self.assertNotEqual(_test, '12:34:56:78:90:ab')

    def test_mac_map_obfuscate_valid_v6(self):
        _test = self.mac_map.get('12:34:56:ff:fe:78:90:ab')
        self.assertNotEqual(_test, '12:34:56:ff:fe:78:90:ab')

    def test_mac_map_obfuscate_valid_v6_quad(self):
        _test = self.mac_map.get('1234:56ff:fe78:90ab')
        self.assertNotEqual(_test, '1234:56ff:fe78:90ab')

    def test_mac_map_skip_ignores(self):
        _test = self.mac_map.get('ff:ff:ff:ff:ff:ff')
        self.assertEqual(_test, 'ff:ff:ff:ff:ff:ff')

    def test_mac_map_avoid_duplicate_obfuscation(self):
        _test = self.mac_map.get('ab:cd:ef:fe:dc:ba')
        _dup = self.mac_map.get(_test)
        self.assertEqual(_test, _dup)

    def test_ip_map_obfuscate_v4_with_cidr(self):
        _test = self.ip_map.get('192.168.1.0/24')
        self.assertNotEqual(_test, '192.168.1.0/24')

    def test_ip_map_obfuscate_no_cidr(self):
        _test = self.ip_map.get('192.168.2.2')
        self.assertNotEqual(_test, '192.168.2.2')

    def test_ip_map_obfuscate_same_subnet(self):
        _net = ip_interface(self.ip_map.get('192.168.3.0/24'))
        _test = ip_interface(self.ip_map.get('192.168.3.1'))
        self.assertTrue(_test.ip in _net.network)

    def test_ip_map_get_same_with_or_without_cidr(self):
        _hostwsub = self.ip_map.get('192.168.4.1/24')
        _hostnosub = self.ip_map.get('192.168.4.1')
        self.assertEqual(_hostwsub.split('/')[0], _hostnosub)

    def test_ip_skip_ignores(self):
        _test = self.ip_map.get('127.0.0.1')
        self.assertEqual(_test, '127.0.0.1')

    def test_hostname_obfuscate_domain_options(self):
        _test = self.host_map.get('www.redhat.com')
        self.assertNotEqual(_test, 'www.redhat.com')

    def test_hostname_obfuscate_same_item(self):
        _test1 = self.host_map.get('example.redhat.com')
        _test2 = self.host_map.get('example.redhat.com')
        self.assertEqual(_test1, _test2)

    def test_hostname_obfuscate_just_domain(self):
        _test = self.host_map.get('redhat.com')
        self.assertNotEqual(_test, 'redhat.com')

    def test_hostname_concurrently_obfuscate_non_loaded_domain(self):
        _test = self.host_map.get('foobar.com')
        self.assertNotEqual(_test, 'foobar.com')

    def test_hostname_concurrently_obfuscate_non_loaded_fqdn(self):
        _test = self.host_map.get('example.foobar.com')
        self.assertNotEqual(_test, 'example.foobar.com')

    def test_keyword_single(self):
        _test = self.kw_map.get('foobar')
        self.assertEqual(_test, 'obfuscatedword0')

    def test_ipv6_obfuscate_global(self):
        _net = '2022:1104:abcd::'
        _ob_net = self.ipv6_map.get(_net)
        self.assertNotEqual(_net, _ob_net, 'Address was unchanged')
        self.assertTrue(_ob_net.startswith('534f'),
                        'Global address does not start with identifier')
        _host = '2022:1104:abcd::1234'
        _ob_host = self.ipv6_map.get(_host)
        self.assertNotEqual(_host, _ob_host, 'Host address was unchanged')
        self.assertTrue(_host.startswith(_net), 'Host address not in network')

    def test_ipv6_link_local(self):
        _test = 'fe80::1234'
        _ob_test = self.ipv6_map.get(_test)
        self.assertTrue(_ob_test.startswith('fe80'),
                        'Link-local identifier not maintained')
        self.assertNotEqual(_test, _ob_test, 'Device address was unchanged')

    def test_ipv6_private(self):
        _net = 'fd00:abcd::'
        _host = 'fd00:abcd::1234'
        _ob_net = self.ipv6_map.get(_net).split('/')[0]
        _ob_host = self.ipv6_map.get(_host)
        self.assertTrue(_ob_net.startswith('fd53'),
                        'Private network does not start with identifier')
        self.assertTrue(_ob_host.startswith(_ob_net),
                        'Private address not in same network')
        self.assertNotEqual(_net, _ob_net, 'Private network was unchanged')

    def test_ipv6_short_network(self):
        _net = 'ff02::'
        _ob_net = self.ipv6_map.get(_net)
        self.assertTrue(_ob_net.startswith(('53', '54')),
                        f'Short network does not start with identifier: '
                        f'{_ob_net}')

    def test_ipv6_consistent_obfuscation(self):
        _test = '2022:1104:abcd::ef09'
        _new = self.ipv6_map.get(_test)
        _second = self.ipv6_map.get(_test)
        self.assertEqual(_new, _second,
                         "Same address produced two different results")

    def test_ipv6_global_no_collision(self):
        """Tests that generating more than 256 global network obfuscations does
        not produce any repeats"""
        _nets = []
        for i in range(1, 300):
            _nets.append(self.ipv6_map.get(f"f{i:03}::abcd").split('::')[0])
        # if there are any duplicates, then the length of the set will not
        # match
        self.assertTrue(len(set(_nets)) == len(_nets),
                        "Duplicate global network obfuscations produced")
        self.assertTrue(_nets[-1].startswith('54'),
                        "First hextet of global network obfuscation over 256"
                        " not expected '54'")


class CleanerParserTests(unittest.TestCase):

    def setUp(self):
        workdir = join(sos.policies.load().get_tmp_dir(None),
                       'sos_avocado_testing')
        self.ip_parser = SoSIPParser(config={}, workdir=workdir)
        self.ipv6_parser = SoSIPv6Parser(config={}, workdir=workdir)
        self.mac_parser = SoSMacParser(config={}, workdir=workdir)
        self.host_parser = SoSHostnameParser(config={}, workdir=workdir)
        self.host_parser.mapping.add('foobar.com')
        self.kw_parser = SoSKeywordParser(config={}, workdir=workdir)
        self.kw_parser.mapping.add('foobar')
        self.kw_parser_none = SoSKeywordParser(config={}, workdir=workdir)
        self.kw_parser.generate_item_regexes()
        self.uname_parser = SoSUsernameParser(config={}, workdir=workdir)
        self.uname_parser.mapping.add('DOMAIN\\myusername')
        self.uname_parser.mapping.add('foo')

    def test_ip_parser_valid_ipv4_line(self):
        line = 'foobar foo 10.0.0.1/24 barfoo bar'
        _test = self.ip_parser.parse_line(line)[0]
        self.assertNotEqual(line, _test)

    def test_ip_parser_invalid_ipv4_line(self):
        # test that invalid IP address is ignored
        line = 'foobar foo 10.1.2.350 barfoo bar'
        _test = self.ip_parser.parse_line(line)[0]
        self.assertEqual(line, _test)

    def test_ip_parser_package_version_line(self):
        line = 'mycoolpackage-1.2.3.4.5'
        _test = self.ip_parser.parse_line(line)[0]
        self.assertEqual(line, _test)

    def test_mac_parser_valid_ipv4_line(self):
        line = 'foobar foo 13:24:35:46:57:68 bar barfoo'
        _test = self.mac_parser.parse_line(line)[0]
        self.assertNotEqual(line, _test)

    def test_mac_parser_valid_ipv6_line(self):
        line = 'foobar foo AA:BB:CC:FF:FE:DD:EE:FF bar barfoo'
        _test = self.mac_parser.parse_line(line)[0]
        self.assertNotEqual(line, _test)

    def test_mac_parser_with_quotes(self):
        line = "foobar foo '12:34:56:78:90:AA' bar barfoo"
        _test = self.mac_parser.parse_line(line)[0]
        self.assertNotEqual(line, _test)
        dline = 'foobar foo "aa:12:bb:34:cc:56" bar barfoo'
        _dtest = self.mac_parser.parse_line(dline)[0]
        self.assertNotEqual(dline, _dtest)

    def test_mac_parser_with_quotes_ipv6(self):
        line = "foobar foo 'FF:EE:DD:FF:FE:CC:BB:AA' bar barfoo"
        _test = self.mac_parser.parse_line(line)[0]
        self.assertNotEqual(line, _test)
        dline = 'foobar foo "DD:EE:FF:FF:FE:BB:CC:AA" bar barfoo'
        _dtest = self.mac_parser.parse_line(dline)[0]
        self.assertNotEqual(dline, _dtest)

    def test_mac_parser_with_quotes_ipv6_quad(self):
        line = "foobar foo 'AABB:CCDD:EEFF:FFAA' bar barfoo"
        _test = self.mac_parser.parse_line(line)[0]
        self.assertNotEqual(line, _test)
        dline = 'foobar foo "AAFF:FFEE:DDCC:BBAA" bar barfoo'
        _dtest = self.mac_parser.parse_line(dline)[0]
        self.assertNotEqual(dline, _dtest)

    def test_hostname_load_hostname_string(self):
        fqdn = 'myhost.subnet.example.com'
        self.host_parser.mapping.add(fqdn)

    def test_hostname_valid_domain_line(self):
        self.host_parser.mapping.add('myhost.subnet.example.com')
        line = 'testing myhost.subnet.example.com in a string'
        _test = self.host_parser.parse_line(line)[0]
        self.assertNotEqual(line, _test)

    def test_hostname_short_name_in_line(self):
        self.host_parser.mapping.add('myhost.subnet.example.com')
        line = 'testing just myhost in a line'
        _test = self.host_parser.parse_line(line)[0]
        self.assertNotEqual(line, _test)

    def test_obfuscate_whole_fqdn_for_given_domainname(self):
        self.host_parser.mapping.add('sostestdomain.domain')
        line = 'let obfuscate soshost.sostestdomain.domain'
        _test = self.host_parser.parse_line(line)[0]
        self.assertFalse('soshost' in _test)
        self.assertFalse('sostestdomain' in _test)

    def test_hostname_no_obfuscate_underscore(self):
        line = 'pam_env.so _why.not_'
        _test = self.host_parser.parse_line(line)[0]
        self.assertEqual(line, _test)

    def test_keyword_parser_valid_line(self):
        line = 'this is my foobar test line'
        _test = self.kw_parser.parse_line(line)[0]
        self.assertNotEqual(line, _test)

    def test_keyword_parser_fullword_only(self):
        line = 'notfoobar and foobars line'
        _test = self.kw_parser.parse_line(line)[0]
        self.assertEqual(line, _test)

    def test_keyword_parser_concurrently_changed(self):
        # a change done to self.kw_parser in setUp will be loaded by
        # self.kw_parser_none as well
        line = 'this is my foobar test line'
        _test = self.kw_parser_none.parse_line(line)[0]
        self.assertNotEqual(line, _test)

    def test_ipv6_parser_strings(self):
        t1 = 'testing abcd:ef01::1234 as a compressed address'
        t2 = 'testing abcd:ef01::5678:1234 as a separate address'
        t3 = 'testing 2607:c540:8c00:3318::34/64 as another address'
        t4 = ('testing 2007:1234:5678:90ab:0987:6543:21fe:dcba as a full '
              'address')
        t1_test = self.ipv6_parser.parse_line(t1)[0]
        t2_test = self.ipv6_parser.parse_line(t2)[0]
        t3_test = self.ipv6_parser.parse_line(t3)[0]
        t4_test = self.ipv6_parser.parse_line(t4)[0]
        self.assertNotEqual(t1, t1_test,
                            f"Parser did not match and obfuscate '{t1}'")
        self.assertNotEqual(t2, t2_test,
                            f"Parser did not match and obfuscate '{t2}'")
        self.assertNotEqual(t3, t3_test,
                            f"Parser did not match and obfuscate '{t3}'")
        self.assertNotEqual(t4, t4_test,
                            f"Parser did not match and obfuscate '{t4}'")

    def test_ipv6_no_match_signature(self):
        modstr = '2D:4F:6E:55:4F:E8:5E:D2:D2:A3:73:62:AB:FD:F9:C5:A5:53:31:93'
        mod_test = self.ipv6_parser.parse_line(modstr)[0]
        self.assertEqual(modstr, mod_test,
                         "Parser matched module signature, and should not")

    def test_ipv6_no_match_log_false_positive(self):
        logln = ('Automatically imported trusted_ca::ca from trusted_ca/ca'
                 ' into production')
        log_test = self.ipv6_parser.parse_line(logln)[0]
        self.assertEqual(logln, log_test,
                         "IPv6 parser incorrectly matched a log line of"
                         "'trusted_ca::ca'")

    def test_ad_username(self):
        line = "DOMAIN\\myusername"
        _test = self.uname_parser.parse_line(line)[0]
        self.assertNotEqual(line, _test)

    def test_too_short_username(self):
        line = "but foo is too short username"
        _test = self.uname_parser.parse_line(line)[0]
        self.assertEqual(line, _test)


class PrepperTests(unittest.TestCase):
    """
    Ensure that the translations for different parser/mapping methods are
    working
    """

    def setUp(self):
        self.prepper = SoSPrepper(SoSOptions())
        self.archive = SoSReportArchive(
            archive_path='tests/test_data/'
                         'sosreport-cleanertest-2021-08-03-qpkxdid.tar.xz',
            keep_binary_files=[],
            tmpdir='/tmp'
        )
        self.host_prepper = HostnamePrepper(SoSOptions(domains=[]))
        self.ipv4_prepper = IPPrepper(SoSOptions())

    def test_parser_method_translation(self):
        self.assertEqual([],
                         self.prepper.get_parser_file_list('hostname', None))

    def test_mapping_method_translation(self):
        self.assertEqual([], self.prepper.get_items_for_map('foobar', None))

    def test_hostname_prepper_map_items(self):
        self.assertEqual(
            ['cleanertest'],
            self.host_prepper.get_items_for_map('hostname', self.archive)
        )

    def test_ipv4_prepper_parser_files(self):
        self.assertEqual(
            ['sos_commands/networking/ip_-o_addr'],
            self.ipv4_prepper.get_parser_file_list('ip', self.archive)
        )

    def test_ipv4_prepper_invalid_parser_files(self):
        self.assertEqual(
            [],
            self.ipv4_prepper.get_parser_file_list('foobar', self.archive)
        )



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

import glob
from sos.report.plugins import Plugin, PluginOpt, IndependentPlugin, CosPlugin


class LogsBase(Plugin):

    short_desc = 'System logs'

    plugin_name = "logs"
    profiles = ('system', 'hardware', 'storage')

    def setup(self):
        rsyslog = 'etc/rsyslog.conf'
        confs = ['/etc/syslog.conf', rsyslog]
        logs = []

        if self.path_exists(rsyslog):
            with open(self.path_join(rsyslog), 'r', encoding='UTF-8') as conf:
                for line in conf.readlines():
                    if line.startswith('$IncludeConfig'):
                        confs += glob.glob(line.split()[1])

        for conf in confs:
            if not self.path_exists(self.path_join(conf)):
                continue
            config = self.path_join(conf)
            logs += self.do_regex_find_all(r"^\S+\s+(-?\/.*$)\s+", config)

        for i in logs:
            if i.startswith("-"):
                i = i[1:]
            if self.path_isfile(i):
                self.add_copy_spec(i)

        self.add_copy_spec([
            "/var/log/auth.log*",
            "/var/log/boot.log",
            "/var/log/dist-upgrade",
            "/var/log/installer",
            "/var/log/kern.log*",
            "/var/log/messages*",
            "/var/log/secure*",
            "/var/log/syslog*",
            "/var/log/udev",
            "/etc/rsyslog.conf",
            "/etc/rsyslog.d",
            "/etc/syslog.conf",
        ])

        self.add_cmd_output("journalctl --disk-usage")
        self.add_dir_listing('/var/log', recursive=True, extra_opts='s')

        # collect journal logs if:
        # - there is some data present, either persistent or runtime only
        # - systemd-journald service exists
        # otherwise fallback to collecting few well known logfiles directly
        journal = any(self.path_exists(self.path_join(p, "log/journal/"))
                      for p in ["/var", "/run"])
        if journal and self.is_service("systemd-journald"):
            self.add_journal(tags=['journal_full', 'journal_all'],
                             priority=100)
            self.add_journal(boot="this", tags='journal_since_boot')
            self.add_journal(boot="last", tags='journal_last_boot')
            if self.get_option("all_logs"):
                self.add_copy_spec([
                    "/var/log/journal/*",
                    "/run/log/journal/*"
                ])

    def postproc(self):
        self.do_path_regex_sub(
            r"/etc/rsyslog*",
            r"(ActionLibdbiPassword |pwd=)(.*)",
            r"\1[********]"
        )


class IndependentLogs(LogsBase, IndependentPlugin):
    """
    This plugin will collect logs traditionally considered to be "system" logs,
    meaning those such as /var/log/messages, rsyslog, and journals that are
    not limited to unit-specific entries.

    Note that the --since option will apply to journal collections by this
    plugin as well as the typical application to log files. Most users can
    expect typical journal collections to include the "full" journal, as well
    as journals limited to this boot and the previous boot.
    """

    plugin_name = "logs"
    profiles = ('system', 'hardware', 'storage')


class CosLogs(LogsBase, CosPlugin):
    option_list = [
        PluginOpt(name="log-days", default=3,
                  desc="the number of days logs to collect")
    ]

    def setup(self):
        super().setup()
        if self.get_option("all_logs"):
            self.add_cmd_output("journalctl -o export")
        else:
            days = self.get_option("log-days", 3)
            self.add_journal(since=f"-{days}days")

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

import re

from sos_tests import StageOneOutputTest


class ReportHelpTest(StageOneOutputTest):
    """Ensure that --help gives the expected output in the expected format

    :avocado: tags=stageone
    """

    sos_cmd = 'report --help'

    def test_all_help_sections_present(self):
        self.assertOutputContains('Global Options:')
        self.assertOutputContains('Report Options:')
        self.assertOutputContains('Cleaner/Masking Options:')


class ReportListPluginsTest(StageOneOutputTest):
    """Ensure that --list-plugins gives the expected output

    :avocado: tags=stageone
    """

    sos_cmd = 'report --list-plugins'

    def test_all_plugin_sections_present(self):
        self.assertOutputContains('plugins are currently enabled:')
        self.assertOutputContains('plugins are currently disabled:')
        self.assertOutputContains('options are available for ALL plugins:')
        self.assertOutputContains('plugin options are available:')
        self.assertOutputContains('Profiles:')

    def test_no_missing_plugin_descriptions(self):
        _out = re.search("The following plugins are currently enabled:(.*?)"
                         "The following plugins are currently disabled:",
                         self.cmd_output.stdout, re.S).group(1).splitlines()
        for ln in _out:
            ln = ln.split()
            # Ignore newlines
            if not ln:
                continue
            assert len(ln) > 1, f"Plugin '{ln[0]}' missing description"

    def test_plugin_formatting(self):
        _out = re.search("The following plugins are currently enabled:(.*?)"
                         "The following plugins are currently disabled:(.*?)"
                         "The following options are available "
                         "for ALL plugins:(.*?)"
                         "The following plugin options are available:(.*?)"
                         "Profiles:",
                         self.cmd_output.stdout, re.S)
        enabled_plugins = _out.group(1).splitlines()
        disabled_plugins = _out.group(2).splitlines()
        options = _out.group(3).splitlines()
        plugin_options = _out.group(4).splitlines()
        for plug in enabled_plugins:
            # Ignore empty lines
            if not plug.strip():
                continue
            self.assertRegex(plug, r' ([\S ]){20} ([\S ])*')
        for plug in disabled_plugins:
            if not plug.strip():
                continue
            self.assertRegex(plug, r' ([\S ]){30} (inactive[ ]{6}) ([\S ])*')
        for opt in options:
            if not opt.strip():
                continue
            self.assertRegex(opt, r' ([\S ]){25} ([\d ]{15}) ([\S ])*')
        for opt in plugin_options:
            if not opt.strip():
                continue
            self.assertRegex(opt, r' ([\S ]){35} ([\S ]{15}) ([\S ])*')


class ReportListPresetsTest(StageOneOutputTest):
    """Ensure that --list-presets gives the expected output

    :avocado: tags=stageone
    """

    sos_cmd = 'report --list-presets'

    def test_presets_formatting(self):
        _out = re.search("The following presets are available:\n\n(.*)",
                         self.cmd_output.stdout, re.S)
        presets = _out.group(1).split("\n\n")
        for preset in presets:
            if not preset.strip():
                continue
            self.assertRegex(
                preset,
                r'[ ]{9}name: .*?\n[ ]{2}description: .*?(\n[ ]{9}note: .*)?'
            )



# Copyright (C) 2014 Red Hat, Inc., Bryn M. Reeves <bmr@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin, DebianPlugin, UbuntuPlugin


class Ctdb(Plugin, DebianPlugin, UbuntuPlugin):

    short_desc = 'Samba Clustered TDB'
    packages = ('ctdb',)
    profiles = ('cluster', 'storage')
    plugin_name = "ctdb"

    def setup(self):
        self.add_copy_spec([
            "/etc/ctdb/ctdb.conf",
            "/etc/ctdb/*.options",
            "/etc/ctdb/nodes",
            "/etc/ctdb/public_addresses",
            "/etc/ctdb/static-routes",
            "/etc/ctdb/multipathd",
            "/var/log/log.ctdb"
        ])

        self.add_cmd_output([
            "ctdb ip",
            "ctdb ping",
            "ctdb status",
            "ctdb ifaces",
            "ctdb listnodes",
            "ctdb listvars",
            "ctdb statistics",
            "ctdb getdbmap",
            "ctdb event script list legacy"
        ])


class RedHatCtdb(Ctdb, RedHatPlugin):
    def setup(self):
        super().setup()
        self.add_copy_spec("/etc/sysconfig/ctdb")

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos_tests import StageTwoReportTest


class UnicodeOpenTest(StageTwoReportTest):
    """This test ensures that we can safely open files that have embedded
    unicode in them, and that those files do not trigger an exception that
    leaves them uncleaned.

    :avocado: tags=stagetwo
    """

    sos_cmd = '--clean -o unicode_test,networking,host'
    files = [('sos-test-unicode.txt', '/tmp/sos-test-unicode.txt')]
    install_plugins = ['unicode_test']

    def test_file_was_collected(self):
        self.assertFileCollected('/tmp/sos-test-unicode.txt')

    def test_file_was_opened(self):
        # if this fails, then we hit an exception when opening the file
        self.assertSosLogContains('Obfuscating tmp/sos-test-unicode.txt')
        self.assertSosLogNotContains('.*Unable to parse.*')

    def test_obfuscation_complete(self):
        # make sure that we didn't stop processing the file after the unicode
        self.assertFileNotHasContent('tmp/sos-test-unicode.txt', '192.168.1.1')



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from os.path import exists
from sos_tests import StageOneReportTest


class LowPrioTest(StageOneReportTest):
    """
    Ensures that --low-priority properly sets our defined constraints on our
    own process

    :avocado: tags=stageone
    """

    sos_cmd = '--low-priority -o kernel'

    def test_ionice_class_set(self):
        _class = self.manifest['components']['report']['priority']['io_class']
        if exists('/usr/bin/ionice'):
            self.assertSosLogContains('Set IO class to idle')
            self.assertEqual(_class, 'idle')
        else:
            self.assertEqual(_class, 'unknown')

    def test_niceness_set(self):
        self.assertSosLogContains('Set niceness of report to 19')
        self.assertEqual(
            self.manifest['components']['report']['priority']['niceness'],
            19
        )



# Copyright (C) 2018 Red Hat, Inc., Jake Hunsaker  <jhunsake@redhat.com>
# Copyright (C) 2013 Red Hat, Inc., Lukas Zapletal <lzap@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from re import match
from shlex import quote
from sos.report.plugins import (Plugin, RedHatPlugin, DebianPlugin,
                                UbuntuPlugin, PluginOpt)


class Foreman(Plugin):

    short_desc = 'Foreman/Satellite systems management'

    plugin_name = 'foreman'
    plugin_timeout = 1800
    profiles = ('sysmgmt',)
    packages = ('foreman',)
    apachepkg = None
    dbhost = "localhost"
    dbpasswd = ""
    env = {"PGPASSWORD": ""}
    option_list = [
        PluginOpt('days', default=14,
                  desc='number of days for dynflow output'),
        PluginOpt('proxyfeatures', default=False,
                  desc='collect features of smart proxies'),
        PluginOpt('puma-gc', default=False,
                  desc='collect Puma GC stats')
    ]
    pumactl = 'pumactl %s -S /usr/share/foreman/tmp/puma.state'

    def setup(self):
        # for external DB, search in /etc/foreman/database.yml for:
        # production:
        # ..
        #   host: some.hostname
        production_scope = False
        try:
            foreman_db = '/etc/foreman/database.yml'
            with open(foreman_db, 'r', encoding='UTF-8') as dfile:
                foreman_lines = dfile.read().splitlines()
            for line in foreman_lines:
                # skip empty lines and lines with comments
                if not line or line[0] == '#':
                    continue
                if line.startswith("production:"):
                    production_scope = True
                    continue
                if production_scope and match(r"\s+host:\s+\S+", line):
                    self.dbhost = line.split()[1]
                if production_scope and match(r"\s+password:\s+\S+", line):
                    self.dbpasswd = line.split()[1]
                # if line starts with a text, it is a different scope
                if not line.startswith(" "):
                    production_scope = False
        except IOError:
            # fallback when the cfg file is not accessible
            pass
        # strip wrapping ".." or '..' around password
        if (self.dbpasswd.startswith('"') and self.dbpasswd.endswith('"')) or \
           (self.dbpasswd.startswith('\'') and self.dbpasswd.endswith('\'')):
            self.dbpasswd = self.dbpasswd[1:-1]
        # set the password to os.environ when calling psql commands to prevent
        # printing it in sos logs
        # we can't set os.environ directly now: other plugins can overwrite it
        self.env = {"PGPASSWORD": self.dbpasswd}

        self.add_file_tags({
            '/var/log/foreman/production.log.*': 'foreman_production_log',
            '/etc/sysconfig/foreman-tasks': 'foreman_tasks_config',
            '/etc/sysconfig/dynflowd': 'foreman_tasks_config',
            '/var/log/httpd/foreman-ssl_access_ssl.log':
                'foreman_ssl_access_ssl_log'
        })

        self.add_forbidden_path([
            "/etc/foreman/*key.pem",
            "/etc/foreman/encryption_key.rb"
        ])

        _hostname = self.exec_cmd('hostname')['output']
        _hostname = _hostname.strip()
        _host_f = self.exec_cmd('hostname -f')['output']
        _host_f = _host_f.strip()

        self.add_copy_spec([
            "/var/log/foreman/production.log",
            f"/var/log/{self.apachepkg}*/foreman-ssl_*_ssl.log"
        ], sizelimit=500)

        # Allow limiting these
        self.add_copy_spec([
            "/etc/foreman/",
            "/etc/sysconfig/foreman",
            "/etc/sysconfig/dynflowd",
            "/etc/default/foreman",
            "/var/log/foreman/dynflow_executor*log*",
            "/var/log/foreman/dynflow_executor*.output*",
            "/var/log/foreman/apipie_cache*.log*",
            "/var/log/foreman/cron*.log*",
            "/var/log/foreman/db_migrate*log*",
            "/var/log/foreman/db_seed*log*",
            "/var/log/foreman/production.log[.-]*",
            "/var/log/foreman-selinux-install.log",
            "/var/log/foreman-proxy-certs-generate*",
            "/usr/share/foreman/Gemfile*",
            f"/var/log/{self.apachepkg}*/foreman*",
            f"/var/log/{self.apachepkg}*/katello-reverse-proxy_error_ssl.log*",
            f"/var/log/{self.apachepkg}*/error_log*",
            f"/etc/{self.apachepkg}*/conf/",
            f"/etc/{self.apachepkg}*/conf.d/",
            f"/var/log/{self.apachepkg}*/katello-reverse-proxy_access_ssl.log*"
        ])

        self.add_cmd_output([
            'foreman-selinux-relabel -nv',
            'passenger-status --show pool',
            'passenger-status --show requests',
            'passenger-status --show backtraces',
            'passenger-memory-stats',
            f'ping -c1 -W1 {_hostname}',
            f'ping -c1 -W1 {_host_f}',
            'ping -c1 -W1 localhost'
        ])

        self.add_dir_listing([
            '/root/ssl-build',
            '/usr/share/foreman/config/hooks',
            '/var/lib/foreman/red_hat_inventory',
        ], recursive=True)

        self.add_cmd_output(
            'qpid-stat -b amqps://localhost:5671 -q \
                    --ssl-certificate=/etc/pki/katello/qpid_router_client.crt \
                    --ssl-key=/etc/pki/katello/qpid_router_client.key \
                    --sasl-mechanism=ANONYMOUS',
            suggest_filename='qpid-stat_-q'
        )
        self.add_cmd_output("hammer ping", tags="hammer_ping", timeout=120)

        # Dynflow Sidekiq
        self.add_cmd_output('systemctl list-units dynflow*',
                            suggest_filename='dynflow_units')
        self.add_service_status('"system-dynflow\\x2dsidekiq.slice"',
                                suggest_filename='dynflow_sidekiq_status')
        self.add_journal(units="dynflow-sidekiq@*")

        # Puma stats & status, i.e. foreman-puma-stats, then
        # pumactl stats -S /usr/share/foreman/tmp/puma.state
        # and optionally also gc-stats
        # if on RHEL with Software Collections, wrap the commands accordingly
        if self.get_option('puma-gc'):
            self.add_cmd_output(self.pumactl % 'gc-stats',
                                suggest_filename='pumactl_gc-stats')
        self.add_cmd_output(self.pumactl % 'stats',
                            suggest_filename='pumactl_stats')
        self.add_cmd_output('/usr/sbin/foreman-puma-status')

        # collect tables sizes, ordered
        _cmd = self.build_query_cmd(
            "SELECT table_name, pg_size_pretty(total_bytes) AS total, "
            "pg_size_pretty(index_bytes) AS INDEX , "
            "pg_size_pretty(toast_bytes) AS toast, pg_size_pretty(table_bytes)"
            " AS TABLE FROM ( SELECT *, "
            "total_bytes-index_bytes-COALESCE(toast_bytes,0) AS table_bytes "
            "FROM (SELECT c.oid,nspname AS table_schema, relname AS "
            "TABLE_NAME, c.reltuples AS row_estimate, "
            "pg_total_relation_size(c.oid) AS total_bytes, "
            "pg_indexes_size(c.oid) AS index_bytes, "
            "pg_total_relation_size(reltoastrelid) AS toast_bytes "
            "FROM pg_class c LEFT JOIN pg_namespace n ON "
            "n.oid = c.relnamespace WHERE relkind = 'r') a) a order by "
            "total_bytes DESC"
        )
        self.add_cmd_output(_cmd, suggest_filename='foreman_db_tables_sizes',
                            env=self.env)
        self.collect_foreman_db()
        self.collect_proxies()

    def collect_foreman_db(self):
        # pylint: disable=too-many-locals
        """ Collect foreman db and dynflow data """
        days = f'{self.get_option("days")} days'
        interval = quote(days)

        # Construct the DB queries, using the days option to limit the range
        # of entries returned

        scmd = (
            "select id,name,value from settings where name not similar to "
            "'%(pass|key|secret)%'"
        )

        dtaskcmd = ('select * from foreman_tasks_tasks where started_at > '
                    f'NOW() - interval {interval} order by started_at asc')

        dyncmd = (
            'select dynflow_execution_plans.* from foreman_tasks_tasks join '
            'dynflow_execution_plans on (foreman_tasks_tasks.external_id = '
            'dynflow_execution_plans.uuid::varchar) where foreman_tasks_tasks.'
            f'started_at > NOW() - interval {interval} order by '
            'foreman_tasks_tasks.started_at asc')

        dactioncmd = (
             'select dynflow_actions.* from foreman_tasks_tasks join '
             'dynflow_actions on (foreman_tasks_tasks.external_id = '
             'dynflow_actions.execution_plan_uuid::varchar) where '
             f'foreman_tasks_tasks.started_at > NOW() - interval {interval} '
             'order by foreman_tasks_tasks.started_at asc')

        dstepscmd = (
            'select dynflow_steps.* from foreman_tasks_tasks join '
            'dynflow_steps on (foreman_tasks_tasks.external_id = '
            'dynflow_steps.execution_plan_uuid::varchar) where '
            f'foreman_tasks_tasks.started_at > NOW() - interval {interval} '
            'order by foreman_tasks_tasks.started_at asc')

        subnetscmd = (
            'SELECT id,network,mask,name,vlanid,gateway,'
            'dns_primary,dns_secondary,boot_mode,ipam,type,description,'
            'mtu,template_id,nic_delay,externalipam_id,'
            'externalipam_group,dhcp_id,tftp_id,dns_id,discovery_id,'
            'httpboot_id,externalipam_id FROM subnets ORDER BY id DESC'
        )

        # counts of fact_names prefixes/types: much of one type suggests
        # performance issues
        factnamescmd = (
            'WITH prefix_counts AS (SELECT split_part(name,\'::\',1) FROM '
            'fact_names) SELECT COUNT(*), split_part AS "fact_name_prefix" '
            'FROM prefix_counts GROUP BY split_part ORDER BY count DESC '
            'LIMIT 100'
        )

        # Populate this dict with DB queries that should be saved directly as
        # postgres formats them. The key will be the filename in the foreman
        # plugin directory, with the value being the DB query to run

        foremandb = {
            'foreman_settings_table': scmd,
            'foreman_schema_migrations': 'select * from schema_migrations',
            'foreman_auth_table': 'select id,type,name,host,port,account,'
                                  'base_dn,attr_login,onthefly_register,tls '
                                  'from auth_sources',
            'foreman_subnets_table': subnetscmd,
            'dynflow_schema_info': 'select * from dynflow_schema_info',
            'audits_table_count': 'select count(*) from audits',
            'logs_table_count': 'select count(*) from logs',
            'fact_names_prefixes': factnamescmd,
            'smart_proxies': 'select name,url,download_policy ' +
                             'from smart_proxies'
        }

        # Same as above, but tasks should be in CSV output

        foremancsv = {
            'foreman_tasks_tasks': dtaskcmd,
            'dynflow_execution_plans': dyncmd,
            'dynflow_actions': dactioncmd,
            'dynflow_steps': dstepscmd,
        }

        for table, val in foremandb.items():
            _cmd = self.build_query_cmd(val)
            self.add_cmd_output(_cmd, suggest_filename=table, timeout=600,
                                sizelimit=100, env=self.env)

        # dynflow* tables on dynflow >=1.6.3 are encoded and hence in that
        # case, psql-msgpack-decode wrapper tool from dynflow-utils (any
        # version) must be used instead of plain psql command
        dynutils = self.is_installed('dynflow-utils')
        for dyn, val in foremancsv.items():
            binary = "psql"
            if dyn != 'foreman_tasks_tasks' and dynutils:
                binary = "/usr/libexec/psql-msgpack-decode"
            _cmd = self.build_query_cmd(val, csv=True, binary=binary)
            self.add_cmd_output(_cmd, suggest_filename=dyn, timeout=600,
                                sizelimit=100, env=self.env)

    def collect_proxies(self):
        """ Collect foreman proxies """
        if self.get_option('proxyfeatures'):
            # get a list of proxy names and URLs, and query for their features
            # store results in smart_proxies_features subdirectory
            _cmd = self.build_query_cmd('select name,url from smart_proxies',
                                        csv=True)
            proxies = self.exec_cmd(_cmd, env=self.env)
            if proxies['status'] == 0:
                # output contains header as the first line, skip it
                for proxy in proxies['output'].splitlines()[1:]:
                    proxy = proxy.split(',')
                    # proxy is now tuple [name, url]
                    _cmd = 'curl -s --key /etc/foreman/client_key.pem ' \
                           '--cert /etc/foreman/client_cert.pem ' \
                           f'{proxy[1]}/v2/features'
                    self.add_cmd_output(_cmd, suggest_filename=proxy[0],
                                        subdir='smart_proxies_features',
                                        timeout=10)

    def build_query_cmd(self, query, csv=False, binary="psql"):
        """
        Builds the command needed to invoke the pgsql query as the postgres
        user.
        The query requires significant quoting work to satisfy both the
        shell and postgres parsing requirements. Note that this will generate
        a large amount of quoting in sos logs referencing the command being run
        """
        if csv:
            query = f"COPY ({query}) TO STDOUT " \
                    "WITH (FORMAT 'csv', DELIMITER ',', HEADER)"
        _dbcmd = "%s --no-password -h %s -p 5432 -U foreman -d foreman -c %s"
        return _dbcmd % (binary, self.dbhost, quote(query))

    def postproc(self):
        self.do_path_regex_sub(
            r"/etc/foreman/(.*)((conf)(.*)?)",
            r"((\:|\s*)(passw|cred|token|secret|key).*(\:\s|=))(.*)",
            r"\1********")
        # yaml values should be alphanumeric
        self.do_path_regex_sub(
            r"/etc/foreman/(.*)((yaml|yml)(.*)?)",
            r"((\:|\s*)(passw|cred|token|secret|key).*(\:\s|=))(.*)",
            r'\1"********"')
        # hide proxy credentials..
        self.do_paths_http_sub([
            '/var/log/foreman/production.log*',
        ])
        # hide proxy credentials from http_proxy setting
        self.do_cmd_output_sub(
            "from settings where",
            r"(http(s)?://)\S+:\S+(@.*)",
            r"\1******:******\3")

# Let the base Foreman class handle the string substitution of the apachepkg
# attr so we can keep all log definitions centralized in the main class


class RedHatForeman(Foreman, RedHatPlugin):

    apachepkg = 'httpd'

    def setup(self):

        self.add_file_tags({
            '/usr/share/foreman/.ssh/ssh_config': 'ssh_foreman_config',
        })

        super().setup()
        self.add_cmd_output('gem list')


class DebianForeman(Foreman, DebianPlugin, UbuntuPlugin):

    apachepkg = 'apache2'

# vim: set et ts=4 sw=4 :



# Copyright (C) 2021 Nadia Pinaeva <npinaeva@redhat.com>
# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

import glob
from sos.report.plugins import Plugin, RedHatPlugin


class OpenshiftOVN(Plugin, RedHatPlugin):
    """This plugin is used to collect OCP 4.x OVN logs.
    """
    short_desc = 'Openshift OVN'
    plugin_name = "openshift_ovn"
    containers = ('ovnkube-master', 'ovnkube-node', 'ovn-ipsec',
                  'ovnkube-controller')
    profiles = ('openshift',)

    def setup(self):

        all_logs = self.get_option("all_logs")

        self.add_copy_spec([
            "/var/lib/ovn/etc/ovnnb_db.db",
            "/var/lib/ovn/etc/ovnsb_db.db",
            "/var/lib/openvswitch/etc/keys"
        ], sizelimit=300)

        # Collect ovn interconnect specific db files if exists.
        self.add_copy_spec([
            "/var/lib/ovn-ic/etc/ovnnb_db.db",
            "/var/lib/ovn-ic/etc/ovnsb_db.db"
        ], sizelimit=300)

        # Collect libovsdb logs in case of ovn interconnect setup.
        if not all_logs:
            self.add_copy_spec([
                "/var/lib/ovn-ic/etc/libovsdb.log",
                "/var/lib/ovn-ic/etc/libovsdb*log.gz"
            ], sizelimit=100)
        else:
            self.add_copy_spec("/var/lib/ovn-ic/etc/libovsdb*log*")

        # The ovn cluster/status is not valid anymore for interconnect setup.
        self.add_cmd_output([
            'ovn-appctl -t /var/run/ovn/ovnnb_db.ctl ' +
            'cluster/status OVN_Northbound',
            'ovn-appctl -t /var/run/ovn/ovnsb_db.ctl ' +
            'cluster/status OVN_Southbound'],
            container='ovnkube-master',
            runtime='crio')
        # We need to determine the actual file name to send
        # to the command
        files = glob.glob("/var/run/ovn/ovn-controller.*.ctl")
        for file in files:
            self.add_cmd_output([
                f"ovs-appctl -t {file} ct-zone-list"],
                container='ovnkube-node',
                runtime='crio')
            self.add_cmd_output([
                f"ovs-appctl -t {file} ct-zone-list"],
                container='ovnkube-controller',
                runtime='crio')
        # Collect ovs ct-zone-list directly on host for interconnect setup.
        files = glob.glob("/var/run/ovn-ic/ovn-controller.*.ctl")
        for file in files:
            self.add_cmd_output([
                f"ovs-appctl -t {file} ct-zone-list'"],
                runtime='crio')
        self.add_cmd_output([
            'ovs-appctl -t ovs-monitor-ipsec tunnels/show',
            'ipsec status',
            'certutil -L -d sql:/etc/ipsec.d'],
            container='ovn-ipsec',
            runtime='crio')



# Copyright (C) 2019 Red Hat, Inc., Miguel Martin <mmartinv@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from datetime import datetime
from sos.report.plugins import Plugin, RedHatPlugin, PluginOpt


class OVirtEngineBackup(Plugin, RedHatPlugin):

    short_desc = 'oVirt Engine database backup'

    packages = ("ovirt-engine-tools-backup",)
    plugin_name = "ovirt_engine_backup"
    option_list = [
        PluginOpt('backupdir', default='/var/lib/ovirt-engine-backup',
                  desc='Directory where backups are generated'),
        PluginOpt('tmpdir', default='/tmp',
                  desc='temp dir to use for engine-backup')
    ]
    profiles = ("virt",)

    def setup(self):
        now = datetime.now().strftime("%Y%m%d%H%M%S")
        backup_filename = self.path_join(
            self.get_option("backupdir"),
            f"engine-db-backup-{now}.tar.gz"
        )
        log_filename = self.path_join(
            self.get_option("backupdir"),
            f"engine-db-backup-{now}.log"
        )
        cmd = ("engine-backup --mode=backup --scope=db"
               f" --file={backup_filename} --log={log_filename}"
               f" --tmpdir={self.get_option('tmpdir')}")
        res = self.collect_cmd_output(cmd, suggest_filename="engine-backup")
        if res['status'] == 0:
            self.add_copy_spec([
                backup_filename,
                log_filename
            ])

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, IndependentPlugin


class Md(Plugin, IndependentPlugin):

    short_desc = 'MD RAID subsystem'

    plugin_name = 'md'
    profiles = ('storage',)
    packages = ('mdadm',)

    def setup(self):
        self.add_cmd_output("mdadm -D /dev/md*")
        mdadm_members = self.exec_cmd("lsblk -o NAME,FSTYPE -r")
        if mdadm_members['status'] == 0:
            for line in mdadm_members['output'].splitlines():
                if 'linux_raid_member' in line:
                    dev = line.split()[0]
                    self.add_cmd_output(f'mdadm -E /dev/{dev}',
                                        tags="mdadm_E")

        self.add_copy_spec([
            "/etc/mdadm.conf",
            "/dev/md/md-device-map",
            "/proc/sys/dev/raid/*",
            "/sys/block/md*/md*"
        ])

        self.add_copy_spec("/proc/mdstat", tags='mdstat')

# vim: set et ts=4 sw=4 :



# Copyright (C) 2021 Red Hat, Inc., Lev Veyde <lveyde@redhat.com>
# Copyright (C) 2018 Red Hat, Inc.,
# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin


class OvirtNode(Plugin, RedHatPlugin):

    short_desc = 'oVirt Node specific information'

    packages = (
        'imgbased',
        'ovirt-node-ng-nodectl',
    )

    plugin_name = 'ovirt_node'
    profiles = ('virt',)

    def setup(self):

        # Add log files
        self.add_copy_spec([
            '/var/log/imgbased.log',
            # Required for node versions < 4.2
            '/tmp/imgbased.log',
        ])

        certificates = [
            '/etc/pki/vdsm/certs/cacert.pem',
            '/etc/pki/vdsm/certs/vdsmcert.pem',
            '/etc/pki/vdsm/libvirt-spice/ca-cert.pem',
            '/etc/pki/vdsm/libvirt-spice/server-cert.pem',
            '/etc/pki/vdsm/libvirt-vnc/ca-cert.pem',
            '/etc/pki/vdsm/libvirt-vnc/server-cert.pem',
        ]

        # Collect runtime info
        self.add_cmd_output([
            'imgbase layout',
            'nodectl --machine-readable check',
            'nodectl info',
        ])

        # Collect certificate info
        self.add_cmd_output([
            f'openssl x509 -in {c} -text -noout' for c in certificates
        ])


# vim: expandtab tabstop=4 shiftwidth=4



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, IndependentPlugin


class Collectl(Plugin, IndependentPlugin):

    short_desc = 'Collectl data'

    plugin_name = "collectl"
    profiles = ('storage', 'system', 'performance')

    packages = ('collectl', )

    def setup(self):
        self.add_copy_spec([
            '/etc/collectl.conf',
            '/var/log/collectl/'
        ])


# vim: set et ts=4 sw=4 :



# Copyright 2020 Red Hat, Inc. Jake Hunsaker <jhunsake@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

import inspect
import importlib
import sys
import os

from collections import OrderedDict
from textwrap import fill
from sos.component import SoSComponent
from sos.policies import import_policy
from sos.report.plugins import Plugin
from sos.utilities import bold, ImporterHelper

try:
    TERMSIZE = min(os.get_terminal_size().columns, 120)
except Exception:
    TERMSIZE = 120


class SoSHelper(SoSComponent):
    """Provide better, more in-depth help for specific parts of sos than is
    provided in either standard --help output or in manpages.
    """

    desc = 'Detailed help infomation'
    configure_logging = False
    load_policy = False
    load_probe = False

    arg_defaults = {
        'topic': ''
    }

    def __init__(self, parser, args, cmdline):
        super().__init__(parser, args, cmdline)
        self.topic = self.opts.topic

    @classmethod
    def add_parser_options(cls, parser):
        parser.usage = 'sos help TOPIC [options]'
        help_grp = parser.add_argument_group(
            'Help Information Options',
            'These options control what detailed information is displayed'
        )
        help_grp.add_argument('topic', metavar='TOPIC', default='', nargs='?',
                              help=('name of the topic or component to show '
                                    'help for'))

    def sanitize_topic_component(self):
        _com = self.opts.topic.split('.')[0]
        _replace = {
            'clean': 'cleaner',
            'mask': 'cleaner',
            'collect': 'collector'
        }
        if _com in _replace:
            self.opts.topic = self.opts.topic.replace(_com, _replace[_com])

    def execute(self):
        if not self.opts.topic:
            self.display_self_help()
            sys.exit(0)

        # standardize the command to the module naming pattern
        self.sanitize_topic_component()

        try:
            klass = self.get_obj_for_topic()
        except Exception as err:
            print(f"Could not load help for '{self.opts.topic}': {err}")
            sys.exit(1)

        if klass:
            try:
                ht = HelpSection()
                klass.display_help(ht)
                ht.display()
            except Exception as err:
                print(f"Error loading help: {err}")
        else:
            print(f"No help section found for '{self.opts.topic}'")

    def get_obj_for_topic(self):
        """Based on the help topic we're after, try to smartly decide which
        object we need to manipulate in order to get help information.
        """
        static_map = {
            'report': 'SoSReport',
            'report.plugins': 'Plugin',
            'cleaner': 'SoSCleaner',
            'collector': 'SoSCollector',
            'collector.transports': 'RemoteTransport',
            'collector.clusters': 'Cluster',
            'policies': 'Policy',
            'upload': 'SoSUpload'
        }

        cls = None

        if self.opts.topic in static_map:
            mod = importlib.import_module('sos.' + self.opts.topic)
            cls = getattr(mod, static_map[self.opts.topic])
        else:
            _help = {
                'report.plugins.': self._get_plugin_variant,
                'policies.': self._get_policy_by_name,
                'collector.transports.': self._get_collect_transport,
                'collector.clusters.': self._get_collect_cluster,
            }
            for _sec, value in _help.items():
                if self.opts.topic.startswith(_sec):
                    cls = value()
                    break
        return cls

    def _get_collect_transport(self):
        from sos.collector.sosnode import TRANSPORTS
        _transport = self.opts.topic.split('.')[-1]
        if _transport in TRANSPORTS:
            return TRANSPORTS[_transport]
        return None

    def _get_collect_cluster(self):
        from sos.collector import SoSCollector
        import sos.collector.clusters
        clusters = SoSCollector._load_modules(sos.collector.clusters,
                                              'clusters')
        for cluster in clusters:
            if cluster[0] == self.opts.topic.split('.')[-1]:
                return cluster[1]
        return None

    def _get_plugin_variant(self):
        mod = importlib.import_module('sos.' + self.opts.topic)
        self.load_local_policy()
        mems = inspect.getmembers(mod, inspect.isclass)
        plugins = [m[1] for m in mems if issubclass(m[1], Plugin)]
        for plugin in plugins:
            if plugin.__subclasses__():
                cls = self.policy.match_plugin(plugin.__subclasses__())
                return cls
        return None

    def _get_policy_by_name(self):
        _topic = self.opts.topic.split('.')[-1]
        # mimic policy loading to discover all policiy classes without
        # needing to manually define each here
        import sos.policies.distros
        _helper = ImporterHelper(sos.policies.distros)
        for mod in _helper.get_modules():
            for policy in import_policy(mod):
                _p = policy.__name__.lower().replace('policy', '')
                if _p == _topic:
                    return policy
        return None

    def display_self_help(self):
        """Displays the help information for this component directly, that is
        help for `sos help`.
        """
        self_help = HelpSection(
            'Detailed help for sos help',
            ('The \'help\' sub-command is used to provide more detailed '
             'information on different sub-commands available to sos as well '
             'as different components at play within those sub-commands.')
        )
        self_help.add_text(
            'SoS - officially pronounced "ess-oh-ess" - is a diagnostic and '
            'supportability utility used by several Linux distributions as an '
            'easy-to-use tool for standardized data collection. The most known'
            f' component of which is {bold("sos report")} (formerly sosreport)'
            ' which is used to collect troubleshooting information into an '
            'archive for review by sysadmins or technical support teams.'
        )

        subsect = self_help.add_section('How to search using sos help')
        usage = bold('$component.$topic.$subtopic')
        subsect.add_text(
            'To get more information on a given topic, use the form '
            f'\'{usage}\'.'
        )

        rep_ex = bold('sos help report.plugins.kernel')
        subsect.add_text(f"For example '{rep_ex}' will provide more "
                         "information on the kernel plugin for the report "
                         "function.")

        avail_help = self_help.add_section('Available Help Sections')
        avail_help.add_text(
            'The following help sections are available. Additional help'
            ' topics and subtopics may be displayed within their respective '
            'help section.\n'
        )

        sections = {
            'report':   'Detailed help on the report command',
            'report.plugins': 'Information on the plugin design of sos',
            'report.plugins.$plugin': 'Information on a specific $plugin',
            'clean':    'Detailed help on the clean command',
            'collect':  'Detailed help on the collect command',
            'upload': 'Detailed help on the upload command',
            'policies': 'How sos operates on different distributions',
        }

        for sect, value in sections.items():
            avail_help.add_text(f"\t{bold(sect):<36}{value}", newline=False)

        self_help.display()


class HelpSection():
    """This class is used to build the output displayed by `sos help` in a
    standard fashion that provides easy formatting controls.
    """

    def __init__(self, title='', content='', indent=''):
        """
        :param title:   The title of the output section, will be prominently
                        displayed
        :type title:    ``str``

        :param content: The text content to be displayed with this section
        :type content:  ``str``

        :param indent:  If the section should be nested, set this to a multiple
                        of 4.
        :type indent:   ``int``
        """
        self.title = title
        self.content = content
        self.indent = indent
        self.sections = OrderedDict()

    def set_title(self, title):
        """Set or override the title for this help section

        :param title:   The name to set for this help section
        :type title:    ``str``
        """
        self.title = title

    def add_text(self, content, newline=True):
        """Add body text to this section. If content for this section already
        exists, append the new ``content`` after a newline.

        :param content:     The text to add to the section
        :type content:      ``str``
        """
        if self.content:
            ln = '\n\n' if newline else '\n'
            content = ln + content
        self.content += content

    def add_section(self, title, content='', indent=''):
        """Add a section of text to the help section that will be displayed
        when the HelpSection object is printed.

        Sections will be printed *in the order added*.

        This will return a subsection object with which block(s) of text may be
        added to the subsection associated with ``title``.

        :param title:   The title of the subsection being added
        :type title:    ``str``

        :param content: The text the new section should contain
        :type content:  ``str``

        :returns:   The newly created subsection for ``title``
        :rtype:     ``HelpSection``
        """
        self._add_section(title, content, indent)
        return self.sections[title]

    def _add_section(self, title, content='', indent=''):
        """Internal method used to add a new subsection to this help output

        :param title:   The title of the subsection being added
        :type title:    ``str`
        """
        if title in self.sections:
            raise Exception('A section with that title already exists')
        self.sections[title] = HelpSection(title, content, indent)

    def display(self):
        """Print the HelpSection contents, including any subsections, to
        console.
        """
        print(fill(
            bold(self.title), width=TERMSIZE, initial_indent=self.indent
        ))
        for ln in self.content.splitlines():
            print(fill(ln, width=TERMSIZE, initial_indent=self.indent))
        for section in self.sections:
            print('')
            self.sections[section].display()



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.


import re

from sos.report.plugins import Plugin, IndependentPlugin


class ArcConf(Plugin, IndependentPlugin):
    """
    The ArcConf plugin is meant for sas adapters, and will collect logs and
    information for each RAID adapter discovered on the system.
    """

    short_desc = 'arcconf Integrated RAID adapter information'

    plugin_name = "arcconf"
    commands = ("arcconf",)

    def setup(self):
        # Get the list of available adapters
        listarcconf = self.collect_cmd_output("arcconf list")

        # Parse the 'arcconf list' output and extract controller IDs.
        # For each Controller ID found in 'arcconf list', add commands
        # for getconfig and GETLOGS
        #
        # Sample 'arcconf list' output:
        #
        # Controller information
        # -------------------------------------------------------------
        #    Controller ID   : Status, Slot, Mode, Name, SerialNumber, WWN
        # -------------------------------------------------------------
        #    Controller 1:   : Optimal, Slot XXXX, XXXX, XXXX, XXXX, XXXX
        # -------------------------------------------------------------
        #    Controller 2:   : Optimal, Slot XXXX, XXXX, XXXX, XXXX, XXXX

        if listarcconf['status'] == 0:
            for line in listarcconf['output'].splitlines():
                try:
                    match = re.match(r"^[\s]*Controller (\d)+", line).group(0)
                    controller_id = match.split()[1]
                    if controller_id:
                        # Add new commands with Controller ID
                        self.add_cmd_output([
                            f"arcconf getconfig {controller_id}",
                            f"arcconf GETLOGS {controller_id} UART",
                        ])
                except AttributeError:
                    continue
# vim: et ts=4 sw=4



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.
import unittest
import os
import tarfile
import tempfile
import shutil

from sos.archive import TarFileArchive
from sos.utilities import tail
from sos.policies import Policy


class TarFileArchiveTest(unittest.TestCase):

    def setUp(self):
        self.tmpdir = tempfile.mkdtemp()
        enc = {'encrypt': False}
        self.tf = TarFileArchive('test', self.tmpdir, Policy(), 1, enc, '/')

    def tearDown(self):
        shutil.rmtree(self.tmpdir)

    def check_for_file(self, filename):
        with tarfile.open(os.path.join(self.tmpdir, 'test.tar.xz')) as rtf:
            rtf.getmember(filename)

    def test_create(self):
        self.tf.finalize('auto')
        self.assertTrue(os.path.exists(os.path.join(self.tmpdir,
                                                    'test.tar.xz')))

    def test_add_file(self):
        self.tf.add_file('tests/unittests/ziptest')
        self.tf.finalize('auto')

        self.check_for_file('test/tests/unittests/ziptest')

    def test_add_node_dev_null(self):
        st = os.lstat('/dev/null')
        dev_maj = os.major(st.st_rdev)
        dev_min = os.minor(st.st_rdev)
        self.tf.add_node('/dev/null', st.st_mode, os.makedev(dev_maj, dev_min))

    # when the string comes from tail() output
    def test_add_string_from_file(self):
        self.copy_strings = []
        with tempfile.NamedTemporaryFile(dir=self.tmpdir, delete=False) \
             as testfile:
            testfile.write(b"*" * 1000)
            testfile.flush()

        self.copy_strings.append((tail(testfile.name, 100), 'string_test.txt'))
        self.tf.add_string(self.copy_strings[0][0], 'tests/string_test.txt')
        self.tf.finalize('auto')

# Since commit 179d9bb add_file does not support recursive directory
# addition. Disable this test for now.
#    def test_add_dir(self):
#        self.tf.add_file('tests/')
#        self.tf.close()
#
#        self.check_for_file('test/tests/ziptest')

    def test_add_renamed(self):
        self.tf.add_file('tests/unittests/ziptest',
                         dest='tests/unittests/ziptest_renamed')
        self.tf.finalize('auto')

        self.check_for_file('test/tests/unittests/ziptest_renamed')

# Since commit 179d9bb add_file does not support recursive directory
# addition. Disable this test for now.
#    def test_add_renamed_dir(self):
#        self.tf.add_file('tests/', 'tests_renamed/')
#        self.tf.close()
#
#        self.check_for_file('test/tests_renamed/ziptest')

    def test_add_string(self):
        self.tf.add_string('this is content', 'tests/string_test.txt')
        self.tf.finalize('auto')

        self.check_for_file('test/tests/string_test.txt')

    def test_get_file(self):
        self.tf.add_string('this is my content', 'tests/string_test.txt')

        afp = self.tf.open_file('tests/string_test.txt')
        self.assertEqual('this is my content', afp.read())

    def test_rewrite_file(self):
        """Test that re-writing a file with add_string() modifies the content.
        """
        self.tf.add_string('this is my content', 'tests/string_test.txt')
        self.tf.add_string('this is my new content', 'tests/string_test.txt')

        afp = self.tf.open_file('tests/string_test.txt')
        self.assertEqual('this is my new content', afp.read())

    def test_make_link(self):
        self.tf.add_file('tests/ziptest')
        self.tf.add_link('tests/ziptest', 'link_name')

        self.tf.finalize('auto')
        self.check_for_file('test/link_name')

    def test_compress(self):
        self.tf.finalize("auto")


if __name__ == "__main__":
    unittest.main()

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, IndependentPlugin


class Cxl(Plugin, IndependentPlugin):
    """This plugin collects data from Compute Express Link (CXL) devices
    """

    short_desc = 'Compute Express Link (CXL)'
    plugin_name = 'cxl'
    profiles = ('storage', 'hardware', 'memory')
    # Utilities can be installed by package or self compiled
    packages = ('cxl-cli', 'daxctl')
    commands = ('cxl', 'daxctl')

    def setup(self):
        """ Use the daxctl-list(1) command to collect disabled, devices,
        mapping, and region information

        Output is JSON formatted
        """
        self.add_cmd_output([
            "daxctl version",
            "daxctl list",
            "daxctl list -iDRM"
        ])

        # Use the cxl-list(1) command to collect data about all CXL devices.
        # Output is JSON formatted.
        self.add_cmd_output([
            "cxl version",
            "cxl list",
            "cxl list -vvv"
        ])

# vim: set et ts=4 sw=4 :



# Copyright (C) 2024 Marcin Wilk <marcin.wilk@canonical.com>
#
# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, IndependentPlugin


class Telegraf(Plugin, IndependentPlugin):

    short_desc = 'Telegraf, the metric collecting tool, plugin'

    plugin_name = "telegraf"
    profiles = ('observability',)
    services = ('telegraf',)
    files = (
        '/etc/telegraf/',
        '/etc/default/telegraf',
    )

    def setup(self):
        # Collect data from 'files' var
        super().setup()

        # collect logs in addition to the above
        if self.get_option("all_logs"):
            self.add_copy_spec([
                "/var/log/telegraf/",
            ])
        else:
            self.add_copy_spec([
                "/var/log/telegraf/*.log",
            ])

    def postproc(self):
        protect_keys = [
            "password",
            "token",
            "pwd",
        ]
        telegraf_path_exps = [
            "/etc/telegraf/*",
            "/etc/default/telegraf",
        ]
        match_exp = fr"(^\s*(.*({'|'.join(protect_keys)}))\s*=\s*)(.*)"

        # Obfuscate passwords and keys
        self.do_path_regex_sub(fr"({'|'.join(telegraf_path_exps)})",
                               match_exp,
                               r"\1*********")

        # Obfuscate certs
        self.do_file_private_sub(telegraf_path_exps[0])


# vim: et ts=4 sw=4



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos_tests import StageOneReportExceptionTest, StageOneReportTest


class InvalidPluginEnabledTest(StageOneReportExceptionTest):
    """
    :avocado: tags=stageone
    """

    sos_cmd = '-o foobar'

    def test_caught_invalid_plugin(self):
        self.assertOutputContains(r'a non-existing plugin \(foobar\)')


class InvalidPluginOptionTest(StageOneReportExceptionTest):
    """
    :avocado: tags=stageone
    """

    sos_cmd = '-o kernel -k kernel.colonel=on'

    def test_caught_invalid_plugin_option(self):
        self.assertOutputContains('no such option "colonel" for plugin '
                                  r'\(kernel\)')


class InvalidReportOptionTest(StageOneReportExceptionTest):
    """
    :avocado: tags=stageone
    """

    sos_cmd = '--magic'

    def test_caught_invalid_option(self):
        self.assertOutputContains(r'unrecognized arguments\: --magic')


class InvalidPluginDisableTest(StageOneReportTest):
    """Ensure passing an invalid plugin name for skipping does not stop the
    execution, see PR#2517

    :avocado: tags=stageone
    """
    sos_cmd = '-n logs,foobar,networking'

    def test_caught_invalid_plugin_name(self):
        self.assertOutputContains("Requested to skip non-existing plugin "
                                  "'foobar'")



# Copyright (C) 2023 Red Hat, Inc., Jose Castillo <jcastillo@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

import logging
try:
    import requests
    REQUESTS_LOADED = True
except ImportError:
    REQUESTS_LOADED = False
import time
from datetime import datetime, timedelta, timezone

from sos.utilities import TIMEOUT_DEFAULT

DEVICE_AUTH_CLIENT_ID = "sos-tools"
GRANT_TYPE_DEVICE_CODE = "urn:ietf:params:oauth:grant-type:device_code"

logger = logging.getLogger("sos")


class DeviceAuthorizationClass:
    """
    Device Authorization Class
    """

    def __init__(self, client_identifier_url, token_endpoint):

        self._access_token = None
        self._access_expires_at = None
        self.__device_code = None

        self.client_identifier_url = client_identifier_url
        self.token_endpoint = token_endpoint
        self._use_device_code_grant()

    def _use_device_code_grant(self):
        """
        Start the device auth flow. In the future we will
        store the tokens in an in-memory keyring.

        """

        self._request_device_code()
        print(
            "Please visit the following URL to authenticate this"
            f" device: {self._verification_uri_complete}"
        )
        self.poll_for_auth_completion()

    def _request_device_code(self):
        """
        Initialize new Device Authorization Grant attempt by
        requesting a new device code.

        """
        data = f"client_id={DEVICE_AUTH_CLIENT_ID}"
        headers = {'content-type': 'application/x-www-form-urlencoded'}
        if not REQUESTS_LOADED:
            raise Exception("python3-requests is not installed and is required"
                            " for obtaining device auth token.")
        try:
            res = requests.post(
                self.client_identifier_url,
                data=data,
                headers=headers,
                timeout=TIMEOUT_DEFAULT)
            res.raise_for_status()
            response = res.json()
            self._user_code = response.get("user_code")
            self._verification_uri = response.get("verification_uri")
            self._interval = response.get("interval")
            self.__device_code = response.get("device_code")
            self._verification_uri_complete = response.get(
                "verification_uri_complete")
        except requests.HTTPError as e:
            raise requests.HTTPError("HTTP request failed "
                                     "while attempting to acquire the tokens."
                                     f"Error returned was {res.status_code} "
                                     f"{e}")

    def poll_for_auth_completion(self):
        """
        Continuously poll OIDC token endpoint until the user is successfully
        authenticated or an error occurs.

        """
        token_data = {'grant_type': GRANT_TYPE_DEVICE_CODE,
                      'client_id': DEVICE_AUTH_CLIENT_ID,
                      'device_code': self.__device_code}

        if not REQUESTS_LOADED:
            raise Exception("python3-requests is not installed and is required"
                            " for obtaining device auth token.")
        while self._access_token is None:
            time.sleep(self._interval)
            try:
                check_auth_completion = requests.post(self.token_endpoint,
                                                      data=token_data,
                                                      timeout=TIMEOUT_DEFAULT)

                status_code = check_auth_completion.status_code

                if status_code == 200:
                    logger.info("The SSO authentication is successful")
                    self._set_token_data(check_auth_completion.json())
                if status_code not in [200, 400]:
                    raise Exception(status_code, check_auth_completion.text)
                if status_code == 400 and \
                    check_auth_completion.json()['error'] not in \
                        ("authorization_pending", "slow_down"):
                    raise Exception(status_code, check_auth_completion.text)
            except requests.exceptions.RequestException as e:
                logger.error(f"Error was found while posting a request: {e}")

    def _set_token_data(self, token_data):
        """
        Set the class attributes as per the input token_data received.
        In the future we will persist the token data in a local,
        in-memory keyring, to avoid visting the browser frequently.
        :param token_data: Token data containing access_token, refresh_token
        and their expiry etc.
        """
        self._access_token = token_data.get("access_token")
        self._access_expires_at = datetime.now(timezone.utc) + \
            timedelta(seconds=token_data.get("expires_in"))
        self._refresh_token = token_data.get("refresh_token")
        self._refresh_expires_in = token_data.get("refresh_expires_in")
        if self._refresh_expires_in == 0:
            self._refresh_expires_at = datetime.max
        else:
            self._refresh_expires_at = datetime.now(timezone.utc) + \
                timedelta(seconds=self._refresh_expires_in)

    def get_access_token(self):
        """
        Get the valid access_token at any given time.
        :return: Access_token
        :rtype: string
        """
        if self.is_access_token_valid():
            return self._access_token
        if self.is_refresh_token_valid():
            self._use_refresh_token_grant()
            return self._access_token
        self._use_device_code_grant()
        return self._access_token

    def is_access_token_valid(self):
        """
        Check the validity of access_token. We are considering it invalid 180
        sec. prior to it's exact expiry time.
        :return: True/False

        """
        return self._access_token and self._access_expires_at and \
            self._access_expires_at - timedelta(seconds=180) > \
            datetime.now(timezone.utc)

    def is_refresh_token_valid(self):
        """
        Check the validity of refresh_token. We are considering it invalid
        180 sec. prior to it's exact expiry time.

        :return: True/False

        """
        return self._refresh_token and self._refresh_expires_at and \
            self._refresh_expires_at - timedelta(seconds=180) > \
            datetime.now(timezone.utc)

    def _use_refresh_token_grant(self, refresh_token=None):
        """
        Fetch the new access_token and refresh_token using the existing
        refresh_token and persist it.
        :param refresh_token: optional param for refresh_token

        """
        if not REQUESTS_LOADED:
            raise Exception("python3-requests is not installed and is required"
                            " for obtaining device auth token.")
        refresh_token_data = {'client_id': DEVICE_AUTH_CLIENT_ID,
                              'grant_type': 'refresh_token',
                              'refresh_token': self._refresh_token if not
                              refresh_token else refresh_token}

        refresh_token_res = requests.post(self.token_endpoint,
                                          data=refresh_token_data,
                                          timeout=TIMEOUT_DEFAULT)

        if refresh_token_res.status_code == 200:
            self._set_token_data(refresh_token_res.json())

        elif refresh_token_res.status_code == 400 and 'invalid' in\
                refresh_token_res.json()['error']:
            logger.warning("Problem while fetching the new tokens from refresh"
                           f" token grant - {refresh_token_res.status_code} "
                           f"{refresh_token_res.json()['error']}."
                           " New Device code will be requested !")
            self._use_device_code_grant()
        else:
            raise Exception(
                "Something went wrong while using the "
                "Refresh token grant for fetching tokens:"
                f" Returned status code {refresh_token_res.status_code}"
                f" and error {refresh_token_res.json()['error']}")



# Copyright (C) 2007 Red Hat, Inc., Kent Lamb <klamb@redhat.com>
# Copyright (C) 2014 Red Hat, Inc., Bryn M. Reeves <bmr@redhat.com>
# Copyright (C) 2021 Red Hat, Inc., Mark Reynolds <mreynolds@redhat.com>
#
# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin


class DirectoryServer(Plugin, RedHatPlugin):

    short_desc = 'Directory Server'

    plugin_name = 'ds'
    profiles = ('identity',)

    files = ('/etc/dirsrv', '/opt/redhat-ds')
    packages = ('redhat-ds-base', 'redhat-ds-7')

    def check_version(self):
        """ Get Directory Server version """
        if self.is_installed("redhat-ds-base") or \
                self.path_exists("/etc/dirsrv"):
            return "ds8"

        if self.is_installed("redhat-ds-7") or \
                self.path_exists("/opt/redhat-ds"):
            return "ds7"

        return False

    def setup(self):
        self.add_forbidden_path([
            "/etc/dirsrv/slapd*/pin.txt",
            "/etc/dirsrv/slapd*/key3.db",
            "/etc/dirsrv/slapd*/pwfile.txt",
            "/etc/dirsrv/slapd*/*passw*",
            "/etc/dirsrv/admin-serv/key[3-4].db",
            "/etc/dirsrv/admin-serv/admpw",
            "/etc/dirsrv/admin-serv/password.conf"
        ])

        try:
            for dsrv in self.listdir("/etc/dirsrv"):
                if dsrv[0:5] == 'slapd':
                    certpath = self.path_join("/etc/dirsrv", dsrv)
                    self.add_cmd_output(f"certutil -L -d {certpath}")
                    self.add_cmd_output(f"dsctl {dsrv} healthcheck")
        except OSError:
            self._log_warn("could not list /etc/dirsrv")

        if not self.check_version():
            self.add_alert("Directory Server not found.")
        elif "ds8" in self.check_version():
            self.add_copy_spec([
                "/etc/dirsrv/slapd*/cert8.db",
                "/etc/dirsrv/slapd*/certmap.conf",
                "/etc/dirsrv/slapd*/dse.ldif",
                "/etc/dirsrv/slapd*/dse.ldif.startOK",
                "/etc/dirsrv/slapd*/secmod.db",
                "/etc/dirsrv/slapd*/schema/*.ldif",
                "/etc/dirsrv/admin-serv",
                "/var/log/dirsrv/*"
            ])
            self.add_file_tags({
                "/var/log/dirsrv/*/access": "dirsrv_access"
            })
        elif "ds7" in self.check_version():
            self.add_copy_spec([
                "/opt/redhat-ds/slapd-*/config",
                "/opt/redhat-ds/slapd-*/logs"
            ])

        self.add_dir_listing("/var/lib/dirsrv/slapd-*/db/*")

    def postproc(self):
        # Example for scrubbing rootpw hash
        #
        # nsslapd-rootpw: AAAAB3NzaC1yc2EAAAADAQABAAABAQDeXYA3juyPqaUuyfWV2HuIM
        # v3gebb/5cvx9ehEAFF2yIKvsQN2EJGTV+hBM1DEOB4eyy/H11NqcNwm/2QsagDB3PVwYp
        # 9VKN3BdhQjlhuoYKhLwgtYUMiGL8AX5g1qxjirIkTRJwjbXkSNuQaXig7wVjmvXnB2o7B
        # zLtu99DiL1AizfVeZTYA+OVowYKYaXYljVmVKS+g3t29Obaom54ZLpfuoGMmyO64AJrWs
        #
        # to
        #
        # nsslapd-rootpw:********

        regexppass = r"(nsslapd-rootpw(\s)*:(\s)*)(\S+)([\r\n]\s.*)*\n"
        regexpkey = r"(nsSymmetricKey(\s)*::(\s)*)(\S+)([\r\n]\s.*)*\n"
        repl = r"\1********\n"
        self.do_path_regex_sub('/etc/dirsrv/*', regexppass, repl)
        self.do_path_regex_sub('/etc/dirsrv/*', regexpkey, repl)

# vim: set et ts=4 sw=4 :



# Copyright (c) 2023 Canonical Ltd., Chi Wai Chan <chiwai.chan@canonical.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

import logging
import json
import re

from sos.collector.clusters import Cluster
from sos.utilities import sos_parse_version
from sos.utilities import sos_get_command_output


def _parse_option_string(strings=None):
    """Parse comma separated string."""
    if not strings:
        return []
    return [string.strip() for string in strings.split(",")]


def _get_index(model_name):
    """Helper function to get Index.

    The reason why we need Index defined in function is because currently
    the collector.__init__ will load all the classes in this module
    and also Index. This will cause bug because it think Index is
    Cluster type. Also We don't want to provide a customized
    filter to remove Index class.
    """

    class Index:
        """Index structure to help parse juju status output.

        Attributes apps, units and machines are dict which key
        is the app/unit/machine name
        and the value is list of targets which format are
        {model_name}:{machine_id}.
        """

        def __init__(self, model_name):
            self.model_name: str = model_name
            self.apps = {}
            self.units = {}
            self.machines = {}
            self.ui_log = logging.getLogger("sos")

        def add_principals(self, juju_status):
            """Adds principal units to index."""
            for app, app_info in juju_status["applications"].items():
                nodes = []
                units = app_info.get("units", {})
                for unit, unit_info in units.items():
                    machine = unit_info["machine"]
                    node = f"{self.model_name}:{machine}"
                    self.units[unit] = [node]
                    self.machines[machine] = [node]
                    nodes.append(node)

                self.apps[app] = nodes

        def add_subordinates(self, juju_status):
            """Add subordinates to index.

            Since subordinates does not have units they need to be
            manually added.
            """
            for app, app_info in juju_status["applications"].items():
                subordinate_to = app_info.get("subordinate-to", [])
                for parent in subordinate_to:
                    # If parent is missing
                    if not self.apps.get(parent):
                        self.ui_log.warning(
                            f"Principal charm {parent} is missing"
                        )
                        continue
                    self.apps[app].extend(self.apps[parent])

                    # If parent's units is missing
                    if "units" not in juju_status["applications"][parent]:
                        self.ui_log.warning(
                            f"Principal charm {parent} is missing units"
                        )
                        continue

                    units = juju_status["applications"][parent]["units"]
                    for _, unit_info in units.items():
                        node = f"{self.model_name}:{unit_info['machine']}"
                        for sub_key, _ in unit_info.get(
                            "subordinates", {}
                        ).items():
                            if sub_key.startswith(app + "/"):
                                self.units[sub_key] = [node]

        def add_machines(self, juju_status):
            """Add machines to index.

            If model does not have any applications it needs to be
            manually added.
            """
            for machine in juju_status["machines"].keys():
                node = f"{self.model_name}:{machine}"
                self.machines[machine] = [node]

    return Index(model_name)


class juju(Cluster):
    """
    The juju cluster profile is intended to be used on juju managed clouds.
    It"s assumed that `juju` is installed on the machine where `sos` is called,
    and that the juju user has superuser privilege to the current controller.

    By default, the sos reports will be collected from all the applications in
    the current model. If necessary, you can filter the nodes by models /
    applications / units / machines with cluster options.

    Example:

    sos collect --cluster-type juju -c "juju.models=sos" -c "juju.apps=a,b,c"

    """

    cmd = "juju"
    cluster_name = "Juju Managed Clouds"
    option_list = [
        ("apps", "", "Filter node list by apps (comma separated regex)."),
        ("units", "", "Filter node list by units (comma separated string)."),
        ("models", "", "Filter node list by models (comma separated string)."),
        (
            "machines",
            "",
            "Filter node list by machines (comma separated string).",
        ),
    ]

    def _cleanup_juju_output(self, output):
        """Remove leading characters before {."""
        return re.sub(
            r"(^[^{]*)(.*)", "\\2", output, count=0, flags=re.MULTILINE
        )

    def _get_model_info(self, model_name):
        """Parse juju status output and return target dict.

        Here are couple helper functions to parse the juju principals units,
        subordinate units and machines.
        """
        juju_status = self._execute_juju_status(model_name)

        index = _get_index(model_name=model_name)
        index.add_principals(juju_status)
        index.add_subordinates(juju_status)
        index.add_machines(juju_status)

        return index

    def _get_juju_version(self):
        """Grab the version of juju"""
        res = sos_get_command_output("juju version")
        return res['output']

    def _execute_juju_status(self, model_name):
        model_option = f"-m {model_name}" if model_name else ""
        format_option = "--format json"
        juju_version = self._get_juju_version()
        if sos_parse_version(juju_version) > sos_parse_version("3"):
            format_option += " --no-color"
        status_cmd = f"{self.cmd} status {model_option} {format_option}"
        res = self.exec_primary_cmd(status_cmd)
        if not res["status"] == 0:
            raise Exception(f"'{status_cmd}' returned error: {res['status']}")
        juju_json_output = self._cleanup_juju_output((res["output"]))

        juju_status = None
        juju_status = json.loads(juju_json_output)
        return juju_status

    def _filter_by_pattern(self, key, patterns, model_info):
        """Filter with regex match."""
        nodes = set()
        for pattern in patterns:
            for param, value in getattr(model_info, key).items():
                if re.match(pattern, param):
                    nodes.update(value or [])
        return nodes

    def _filter_by_fixed(self, key, patterns, model_info):
        """Filter with fixed match."""
        nodes = set()
        for pattern in patterns:
            for param, value in getattr(model_info, key).items():
                if pattern == param:
                    nodes.update(value or [])
        return nodes

    def set_transport_type(self):
        """Dynamically change transport to 'juju'."""
        return "juju"

    def get_nodes(self):
        """Get the machine numbers from `juju status`."""
        models = _parse_option_string(self.get_option("models"))
        apps = _parse_option_string(self.get_option("apps"))
        units = _parse_option_string(self.get_option("units"))
        machines = _parse_option_string(self.get_option("machines"))
        filters = {"apps": apps, "units": units, "machines": machines}

        # Return empty nodes if no model and filter provided.
        if not any(filters.values()) and not models:
            return []

        if not models:
            models = [""]  # use current model by default

        nodes = set()

        for model in models:
            model_info = self._get_model_info(model)
            for key, resource in filters.items():
                # Filter node by different policies
                if key == "apps":
                    _nodes = self._filter_by_pattern(key, resource, model_info)
                else:
                    _nodes = self._filter_by_fixed(key, resource, model_info)
                nodes.update(_nodes)

        return list(nodes)


# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

import random
import os

from string import ascii_uppercase, digits
from time import sleep
from sos_tests import StageOneReportTest, StageTwoReportTest


class LogsPluginTest(StageOneReportTest):
    """Ensure common collections from the `logs` plugin are properly collected

    :avocado: tags=stageone
    """

    sos_cmd = '-o logs --all-logs'

    def test_journalctl_collections(self):
        self.assertFileCollected('sos_commands/logs/journalctl_--disk-usage')
        self.assertFileCollected('sos_commands/logs/journalctl_--no-pager_'
                                 '--boot')

    def test_journal_runtime_collected(self):
        self.assertFileGlobInArchive('/var/log/journal/*')


class JournalSizeLimitTest(StageTwoReportTest):
    """Test that journal size limiting is working and is independent of
    --log-size

    Note: this test will insert over 100MB of garbage into the test system's
    journal

    :avocado: tags=stagetwo
    """

    sos_cmd = '-o logs --journal-size=10 --log-size=5'
    sos_timeout = 500
    packages = {
        'rhel': ['python3-systemd'],
        'ubuntu': ['python3-systemd']
    }

    def pre_sos_setup(self):
        # if the journal is already over our size limit, don't write anything
        # new to it
        from systemd import journal  # pylint: disable=import-error
        _reader = journal.Reader()
        _size = _reader.get_usage() / 1024 / 1024
        if _size > 30:
            return
        # write 20MB at a time to side-step rate/size limiting on some distros
        # write over 20MB to ensure we will actually size limit inside sos,
        # allowing for any compression or de-dupe systemd does
        sosfd = journal.stream('sos-testing')
        rsize = 10 * 1048576
        for _ in range(2):
            # generate 10MB, write it, then write it in reverse.
            # Spend less time generating new strings
            rand = ''.join(
                random.choice(ascii_uppercase + digits) for _ in range(rsize))
            sosfd.write(rand + '\n')
            # sleep to avoid burst rate-limiting
            sleep(5)
            sosfd.write(rand[::-1] + '\n')

    def test_journal_size_limit(self):
        journ = 'sos_commands/logs/journalctl_--no-pager'
        self.assertFileCollected(journ)
        jsize = os.stat(self.get_name_in_archive(journ)).st_size
        assert \
            jsize <= 20971520, \
            f"Collected journal is larger than 20MB (size: {jsize})"

    def test_journal_tailed_and_linked(self):
        tailed = self.get_name_in_archive('sos_strings/logs/'
                                          'journalctl_--no-pager.tailed')
        self.assertFileExists(tailed)
        journ = self.get_name_in_archive('sos_commands/logs/'
                                         'journalctl_--no-pager')
        assert \
            os.path.islink(journ), \
            "Journal in sos_commands/logs is not a symlink"



# Copyright (C) 2019 Red Hat, Inc. Jake Hunsaker <jhunsake@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin


class Qt(Plugin, RedHatPlugin):

    short_desc = 'QT widget toolkit'

    plugin_name = 'qt'
    packages = ('qt', )

    def setup(self):
        self.add_env_var([
            'QT_IM_MODULE',
            'QTDIR',
            'QTLIB',
            'QT_PLUGIN_PATH',
            'IMSETTINGS_MODULE'
        ])

# vim: set et ts=4 sw=4 :



# Copyright (C) 2024 Alejandro Santoyo <alejandro.santoyo@canonical.com>
#
# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

import os
from sos.report.plugins import Plugin, UbuntuPlugin


class Kafka(Plugin, UbuntuPlugin):
    """
    This plugin collects log and configuration files, and also basic
    installation information (e.g., `snap info`) for Apache Kafka.
    """

    short_desc = 'Apache Kafka plugin'
    plugin_name = 'kafka'
    profiles = ('services',)
    packages = ('charmed-kafka',)
    services = ('kafka',)

    def setup(self):
        log_file_pattern = "*.log*" if self.get_option("all_logs") else "*.log"

        if self.is_snap:
            self.add_cmd_output('snap info charmed-kafka')

            log_path = "/var/snap/charmed-kafka/common/var/log/kafka/"
            config_path = "/var/snap/charmed-kafka/current/etc/kafka/"
        else:
            log_path = "/usr/local/kafka/logs/"
            config_path = "/usr/local/kafka/config/"

        self.add_copy_spec([
            log_path + log_file_pattern,
            config_path,
        ])

    def postproc(self):
        protect_keys = ["password", "username",]
        config_path = (
            "/var/snap/charmed-kafka/current/etc/kafka/"
            if self.is_snap
            else "/usr/local/kafka/config/"
        )

        # get the absolute paths for all files in the config dir
        # (considering nested directories) and run do_path_regex_sub()
        # on each file to obfuscate the keys in protect_keys
        regexp = fr"(^\s*({'|'.join(protect_keys)})\s*=\s*)(.*)"
        for root, _, files in os.walk(config_path):
            for file in files:
                self.do_path_regex_sub(os.path.join(root, file),
                                       regexp, r"\1*********")

# vim: set et ts=4 sw=4 :



# Copyright 2020 Red Hat, Inc. Jake Hunsaker <jhunsake@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

import re
from sos.cleaner.parsers import SoSCleanerParser
from sos.cleaner.mappings.hostname_map import SoSHostnameMap


class SoSHostnameParser(SoSCleanerParser):

    name = 'Hostname Parser'
    map_file_key = 'hostname_map'
    regex_patterns = [
        r'(((\b|_)[a-zA-Z0-9-\.]{1,200}\.[a-zA-Z]{1,63}(\b|_)))'
    ]

    def __init__(self, config, workdir, skip_cleaning_files=[]):
        self.mapping = SoSHostnameMap(workdir)
        super().__init__(config, skip_cleaning_files)

    def parse_line(self, line):
        """This will be called for every line in every file we process, so that
        every parser has a chance to scrub everything.

        We are overriding parent method since we need to swap ordering of
        _parse_line_with_compiled_regexes and _parse_line calls.
        """
        count = 0
        for skip_pattern in self.skip_line_patterns:
            if re.match(skip_pattern, line, re.I):
                return line, count
        line, _count = self._parse_line(line)
        count += _count
        if self.compile_regexes:
            line, _rcount = self._parse_line_with_compiled_regexes(line)
            count += _rcount
        return line, count



# Copyright (C) 2014 Red Hat, Inc., Bryn M. Reeves <bmr@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

import re
import yaml
from sos.report.plugins import Plugin, RedHatPlugin, DebianPlugin, UbuntuPlugin


class MongoDb(Plugin):

    short_desc = 'MongoDB document database'

    plugin_name = 'mongodb'
    profiles = ('services',)

    var_puppet_gen = "/var/lib/config-data/puppet-generated/mongodb"

    files = (
        '/etc/mongodb.conf',
        var_puppet_gen + '/etc/mongod.conf'
    )

    db_folder = "/var/lib/mongodb"

    def setup(self):
        self.add_copy_spec([
            "/etc/mongodb.conf",
            self.var_puppet_gen + "/etc/",
            self.var_puppet_gen + "/etc/systemd/system/mongod.service.d/",
            "/var/log/mongodb/mongodb.log",
            "/var/lib/mongodb/mongodb.log*"
        ])
        self.add_cmd_output(f"du -sh {self.db_folder}/")

    def postproc(self):
        for file in ["/etc/mongodb.conf",
                     self.var_puppet_gen + "/etc/mongodb.conf"]:
            self.do_file_sub(
                file,
                r"(mms-token)\s*=\s*(.*)",
                r"\1 = ********"
            )


class RedHatMongoDb(MongoDb, RedHatPlugin):

    packages = (
        'mongodb-server',
        'rh-mongodb32-mongodb-server',
        'rh-mongodb34-mongodb-server',
        'rh-mongodb36-mongodb-server'
    )

    def setup(self):
        super().setup()
        self.add_copy_spec([
            "/etc/sysconfig/mongodb",
            "/etc/rh-mongodb*-mongo*.conf",
            "/etc/opt/rh/rh-mongodb*/mongo*.conf",
            "/var/opt/rh/rh-mongodb*/log/mongodb/mongod.log"
        ])


class UbuntuMongodb(MongoDb, DebianPlugin, UbuntuPlugin):

    packages = (
        'mongodb-server',
        'mongodb-server-core',
        'juju-db',
    )

    files = (
        '/var/lib/juju/db',
        '/var/snap/juju-db/current/db',
    )

    services = (
        'juju-db',
        'mongodb',
    )

    def setup(self):
        if get_juju_info := self.path_exists('/var/lib/juju/db'):
            self.db_folder = "/var/lib/juju/db"
        elif get_juju_info := self.path_exists('/var/snap/juju-db/curent/db'):
            self.db_folder = "/var/snap/juju-db/current/db"

        super().setup()

        if get_juju_info:
            for the_dir in self.listdir('/var/lib/juju/agents'):
                if re.search('machine-*', the_dir):
                    username = the_dir
                    with open(f'/var/lib/juju/agents/{username}/agent.conf',
                              'r', encoding='UTF-8') as f:
                        data = yaml.safe_load(f)
                        password = data['statepassword']

                        self._capture_db_data(username, password)
                    break

    def _capture_db_data(self, username, password):
        if self.path_exists("/usr/bin/mongo"):
            client = "/usr/bin/mongo"
        elif self.path_exists("/usr/lib/juju/mongo*/bin/mongo"):
            client = "/usr/lib/juju/mongo*/bin/mongo"
        else:
            client = "/snap/bin/juju-db.mongo"

        cmds_to_check = [
            'db.hostInfo()',
            'db.getCollectionInfos()',
            'db.getCollectionNames()',
            'db.getProfilingStatus()',
            'db.replicationInfo()',
            'db.serverStatus()',
            'db.stats()',
            'rs.conf()',
            'rs.status()',
        ]

        for cmd in cmds_to_check:
            self.add_cmd_output(
                f"{client} 127.0.0.1:37017/juju --authenticationDatabase admin"
                f" --ssl --sslAllowInvalidCertificates --username {username}"
                f" --password {password} --eval {cmd}",
                suggest_filename=cmd, subdir="db_commands"
            )


# vim: set et ts=4 sw=4 :



# Copyright (c) 2025 Rudnei Bertol Jr <rudnei@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin


class AAPEDAControllerPlugin(Plugin, RedHatPlugin):
    short_desc = 'AAP EDA Controller plugin'
    plugin_name = 'aap_eda'
    profiles = ('sysmgmt', 'ansible')
    packages = ('automation-eda-controller',
                'automation-eda-controller-server')

    def setup(self):
        if self.get_option("all_logs"):
            self.add_copy_spec([
                "/etc/ansible-automation-platform/",
                "/var/log/ansible-automation-platform/eda/",
                "/var/log/nginx/automationedacontroller.access.log*",
                "/var/log/nginx/automationedacontroller.error.log*",
            ])
        else:
            self.add_copy_spec([
                "/etc/ansible-automation-platform/",
                "/var/log/ansible-automation-platform/eda/*.log",
                "/var/log/nginx/automationedacontroller.access.log",
                "/var/log/nginx/automationedacontroller.error.log",
            ])

        self.add_forbidden_path([
            "/etc/ansible-automation-platform/eda/SECRET_KEY",
            "/etc/ansible-automation-platform/eda/server.cert",
            "/etc/ansible-automation-platform/eda/server.key",
        ])

        self.add_cmd_output("aap-eda-manage --version")
        self.add_dir_listing([
            "/etc/ansible-automation-platform/",
            "/var/log/ansible-automation-platform/",
        ], recursive=True)

        self.add_cmd_output("su - eda -c 'export'",
                            suggest_filename="eda_export")

    def postproc(self):
        self.do_path_regex_sub(
            "/etc/ansible-automation-platform/eda/environment",
            r"(EDA_SECRET_KEY|EDA_DB_PASSWORD)(\s*)(=|:)(\s*)(.*)",
            r'\1\2\3\4********')

# vim: set et ts=4 sw=4 :



# Copyright (C) 2014 Red Hat, Inc. Neependra Khare <nkhare@redhat.com>
# Copyright (C) 2014 Red Hat, Inc. Bryn M. Reeves <bmr@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from fnmatch import translate
import re
import json
import os
from sos.report.plugins import (Plugin, RedHatPlugin, DebianPlugin,
                                UbuntuPlugin, PluginOpt)


KUBE_PACKAGES = (
    'kubelet',
    'kubernetes',
)

KUBE_SVCS = (
    'kubelet',
    'kube-apiserver',
    'kube-proxy',
    'kube-scheduler',
    'kube-controller-manager',
)

KUBECONFIGS = (
    '/etc/kubernetes/admin.conf',
)


class Kubernetes(Plugin):

    short_desc = 'Kubernetes container orchestration platform'

    plugin_name = "kubernetes"
    profiles = ('container',)
    plugin_timeout = 1200

    config_files = [
        "/etc/kubernetes",
        "/run/flannel",
        "/var/lib/kubelet/config.yaml",
        "/var/lib/kubelet/kubeadm-flags.env",
        "/var/lib/kubelet/*_manager_state",
    ]
    forbidden_paths = [
        "/etc/kubernetes/pki",
    ]
    kube_system_logs = [
        "/var/log/pods/kube-system_etcd-*",
        "/var/log/pods/kube-system_kube-apiserver-*",
        "/var/log/pods/kube-system_kube-controller-manager-*",
        "/var/log/pods/kube-system_kube-scheduler-*",
    ]
    resources = [
        'events',
        'deployments',
        'ingresses',
        'pods',
        'pvc',
        'services',
        'daemonsets',
        'replicasets',
        'endpoints',
        'statefulsets',
        'configmaps',
        'serviceaccounts',
        'secrets',
        'jobs',
        'cronjobs',
        'clusterroles',
        'clusterrolebindings',
        'limitranges',
        'resourcequotas',
    ]

    # these are not namespaced, must pull separately.
    global_resources = [
        'sc',
        'pv',
        'roles',
        'rolebindings',
    ]

    option_list = [
        PluginOpt('all', default=False,
                  desc='collect all namespace output separately'),
        PluginOpt('describe', default=False,
                  desc='collect describe output of all resources'),
        PluginOpt('kubelogs', default=False,
                  desc='copy some kube-system pod logs without using the API'),
        PluginOpt('podlogs', default=False,
                  desc='capture stdout/stderr logs from pods using the API'),
        PluginOpt('podlogs-filter', default='', val_type=str,
                  desc='only collect logs from pods matching this pattern')
    ]

    kube_cmd = "kubectl"

    def set_kubeconfig(self):
        if os.environ.get('KUBECONFIG'):
            return
        for _kconf in self.files:
            if self.path_exists(_kconf):
                self.kube_cmd += f" --kubeconfig={_kconf}"
                break

    def check_is_master(self):
        """ Check if this is the master node """
        return any(self.path_exists(f) for f in self.files)

    def setup(self):
        self.add_copy_spec(self.config_files)

        self.add_forbidden_path(self.forbidden_paths)

        if self.get_option('kubelogs'):
            self.add_copy_spec(self.kube_system_logs)

        self.add_env_var([
            'KUBECONFIG',
            'KUBERNETES_HTTP_PROXY',
            'KUBERNETES_HTTPS_PROXY',
            'KUBERNETES_NO_PROXY',
        ])

        # We can only grab kubectl output from the master
        if not self.check_is_master():
            return

        for subcmd in ['version', 'config view']:
            self.add_cmd_output(
                f'{self.kube_cmd} {subcmd}',
                subdir='cluster-info'
            )

        if self.get_option('all'):
            self.add_cmd_output([
                f"{self.kube_cmd} get -o json {res}"
                for res in self.global_resources
            ], subdir='cluster-info')
        else:
            self.add_cmd_output([
                f"{self.kube_cmd} get {res}"
                for res in self.global_resources
            ], subdir='cluster-info')

        # Get detailed node information
        nodes = self.collect_cmd_output(f"{self.kube_cmd} get nodes",
                                        subdir='cluster-info')
        if nodes['status'] == 0 and self.get_option('describe'):
            for line in nodes['output'].splitlines()[1:]:
                # find first word in the line and ignore empty+blank lines
                words = line.split()
                if not words:
                    continue
                node = words[0]
                self.add_cmd_output(
                    f"{self.kube_cmd} describe node {node}",
                    subdir='cluster-info'
                )

        self.add_cmd_output([
            f"{self.kube_cmd} get -o json nodes",
        ], subdir='cluster-info')

        # Also collect master metrics
        self.add_cmd_output(
            f"{self.kube_cmd} get --raw /metrics",
            subdir='cluster-info'
        )

        # CNV is not part of the base installation, but can be added
        if self.is_installed('kubevirt-virtctl'):
            self.resources.extend(['vms', 'vmis'])
            self.add_cmd_output('virtctl version')

        self.collect_per_resource_details()
        self.collect_all_resources()

    def collect_per_resource_details(self):
        """ Collect details about each resource in all namespaces """
        # get all namespaces in use
        kns = self.collect_cmd_output(f'{self.kube_cmd} get namespaces',
                                      subdir='cluster-info')
        # namespace is the 1st word on line, until the line has spaces only
        kn_output = kns['output'].splitlines()[1:]
        knsps = [n.split()[0] for n in kn_output if n and len(n.split())]

        for nspace in knsps:
            knsp = f'--namespace={nspace}'
            if self.get_option('all'):
                k_cmd = f'{self.kube_cmd} get -o json {knsp}'

                for res in self.resources:
                    self.add_cmd_output(
                        f'{k_cmd} {res}',
                        subdir=f'cluster-info/{nspace}'
                    )

            if self.get_option('describe'):
                # need to drop json formatting for this
                k_cmd = f'{self.kube_cmd} {knsp}'
                for res in self.resources:
                    if res == 'events':
                        continue
                    ret = self.exec_cmd(f'{k_cmd} get {res}')
                    if ret['status'] == 0:
                        k_list = [k.split()[0] for k in
                                  ret['output'].splitlines()[1:]]
                        for item in k_list:
                            k_cmd = f'{self.kube_cmd} {knsp}'
                            self.add_cmd_output(
                                f'{k_cmd} describe {res} {item}',
                                subdir=f'cluster-info/{nspace}/{res}'
                            )

            if self.get_option('podlogs'):
                self._get_pod_logs(knsp)

    def _get_pod_logs(self, namespace):
        k_cmd = f'{self.kube_cmd} get -o json {namespace}'
        ret = self.exec_cmd(f'{k_cmd} pods')
        if ret['status'] == 0:
            pods = json.loads(ret['output'])
            # allow shell-style regex
            reg = (translate(self.get_option('podlogs-filter')) if
                   self.get_option('podlogs-filter') else None)
            for pod in pods["items"]:
                if reg and not re.match(reg, pod["metadata"]["name"]):
                    continue
                _subdir = (f'cluster-info/'
                           f'{pod["metadata"]["namespace"]}/podlogs/'
                           f'{pod["metadata"]["name"]}')
                if "containers" in pod["spec"]:
                    for cont in pod["spec"]["containers"]:
                        pod_name = pod["metadata"]["name"]
                        cont_name = cont["name"]
                        self.add_cmd_output(
                            f'{self.kube_cmd} {namespace} logs '
                            f'{pod_name} -c {cont_name}',
                            subdir=_subdir
                        )
                if "initContainers" in pod["spec"]:
                    for cont in pod["spec"]["initContainers"]:
                        pod_name = pod["metadata"]["name"]
                        cont_name = cont["name"]
                        self.add_cmd_output(
                            f'{self.kube_cmd} {namespace} logs '
                            f'{pod_name} -c {cont_name}',
                            subdir=_subdir
                        )

    def collect_all_resources(self):
        """ Collect details about all resources """
        if not self.get_option('all'):
            k_cmd = f'{self.kube_cmd} get --all-namespaces=true'
            for res in self.resources:
                self.add_cmd_output(
                    f'{k_cmd} {res}',
                    subdir='cluster-info'
                )

    def postproc(self):
        # First, clear sensitive data from the json output collected.
        # This will mask values when the "name" looks susceptible of
        # values worth obfuscating, i.e. if the name contains strings
        # like "pass", "pwd", "key" or "token"
        env_regexp = r'(?P<var>{\s*"name":\s*[^,]*' \
            r'(pass|pwd|key|token|cred|PASS|PWD|KEY)[^,]*,\s*"value":)[^}]*'
        self.do_cmd_output_sub(self.kube_cmd, env_regexp,
                               r'\g<var> "********"')

        # Next, we need to handle the private keys and certs in some
        # output that is not hit by the previous iteration.
        self.do_cmd_private_sub(self.kube_cmd)

        pathexp = fr'^({"|".join(self.config_files)})'
        self.do_file_private_sub(pathexp)

        # clear base64 encoded PEM from kubeconfigs files
        regexp = r'LS0tLS1CRUdJ[A-Za-z0-9+/=]+'
        subst = '***** SCRUBBED BASE64 PEM *****'
        pathexp = fr'^({"|".join(list(self.files)+self.config_files)})'
        self.do_path_regex_sub(pathexp, regexp, subst)


class RedHatKubernetes(Kubernetes, RedHatPlugin):

    packages = KUBE_PACKAGES

    files = KUBECONFIGS

    services = KUBE_SVCS

    def check_enabled(self):
        # do not run at the same time as the openshift plugin
        if self.is_installed("openshift-hyperkube"):
            return False
        return super().check_enabled()

    def setup(self):
        self.set_kubeconfig()
        super().setup()


class UbuntuKubernetes(Kubernetes, UbuntuPlugin, DebianPlugin):

    packages = KUBE_PACKAGES + (
        'k8s',
    )

    files = KUBECONFIGS + (
        '/root/cdk/cdk_addons_kubectl_config',
        '/var/snap/microk8s/current/credentials/client.config',
    )

    services = KUBE_SVCS + (
        'snap.kubelet.daemon',
        'snap.kube-apiserver.daemon',
        'snap.kube-proxy.daemon',
        'snap.kube-scheduler.daemon',
        'snap.kube-controller-manager.daemon',
        # CDK
        'cdk.master.auth-webhook',
    )

    def setup(self):
        self.set_kubeconfig()

        if self.is_installed('microk8s'):
            self.kube_cmd = 'microk8s kubectl'
        elif self.is_installed('k8s'):
            self.kube_cmd = 'k8s kubectl'
            self._canonical_kubernetes()

        self.config_files.extend([
            '/root/cdk/kubelet/config.yaml',
            '/root/cdk/audit/audit-policy.yaml',
        ])
        super().setup()

    def _canonical_kubernetes(self):
        self.add_journal(units="snap.k8s.*")

        k8s_cmd = "k8s"
        k8s_common = "/var/snap/k8s/common"

        k8s_subcmds = [
            'status',
            'get',
        ]

        self.add_copy_spec([
            f"{k8s_common}/args",
            f"{k8s_common}/var/lib/k8s-dqlite/info.yaml",
            f"{k8s_common}/var/lib/k8s-dqlite/cluster.yaml",
            f"{k8s_common}/var/lib/k8sd/state/truststore/k8s.yaml",
            f"{k8s_common}/var/lib/k8sd/state/database/info.yaml",
            f"{k8s_common}/var/lib/k8sd/state/database/cluster.yaml",
            f"{k8s_common}/var/lib/k8sd/state/daemon.yaml",
        ])

        self.add_cmd_output([
            f"{k8s_cmd} {subcmd}" for subcmd in k8s_subcmds
        ])

    def postproc(self):
        super().postproc()

        self.do_file_private_sub(
            "/var/snap/k8s/common/var/lib/k8sd/state/truststore/k8s.yaml")


# vim: et ts=5 sw=4



# Copyright 2023 Red Hat, Inc. Jake Hunsaker <jhunsake@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

import os

from sos.cleaner.preppers import SoSPrepper


class KeywordPrepper(SoSPrepper):
    """
    Prepper to handle keywords passed to cleaner via either the `--keywords`
    or `--keyword-file` options.
    """

    name = 'keyword'

    # pylint: disable=unused-argument
    def _get_items_for_keyword(self, archive):
        items = []
        for kw in self.opts.keywords:
            items.append(kw)
        if self.opts.keyword_file and os.path.exists(self.opts.keyword_file):
            with open(self.opts.keyword_file, 'r', encoding='utf-8') as kwf:
                items.extend(kwf.read().splitlines())

        for item in items:
            self.regex_items['keyword'].add(item)

        return items

# vim: set et ts=4 sw=4 :
